'use strict'

function _typeof(obj) {
  '@babel/helpers - typeof'

  if (typeof Symbol === 'function' && typeof Symbol.iterator === 'symbol') {
    _typeof = function (obj) {
      return typeof obj
    }
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === 'function' && obj.constructor === Symbol && obj !== Symbol.prototype ? 'symbol' : typeof obj
    }
  }

  return _typeof(obj)
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError('Cannot call a class as a function')
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i]
    descriptor.enumerable = descriptor.enumerable || false
    descriptor.configurable = true
    if ('value' in descriptor) descriptor.writable = true
    Object.defineProperty(target, descriptor.key, descriptor)
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps)
  if (staticProps) _defineProperties(Constructor, staticProps)
  return Constructor
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== 'function' && superClass !== null) {
    throw new TypeError('Super expression must either be null or a function')
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true,
    },
  })
  if (superClass) _setPrototypeOf(subClass, superClass)
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf
    ? Object.getPrototypeOf
    : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o)
      }
  return _getPrototypeOf(o)
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf =
    Object.setPrototypeOf ||
    function _setPrototypeOf(o, p) {
      o.__proto__ = p
      return o
    }

  return _setPrototypeOf(o, p)
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === 'undefined' || !Reflect.construct) return false
  if (Reflect.construct.sham) return false
  if (typeof Proxy === 'function') return true

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}))
    return true
  } catch (e) {
    return false
  }
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
  }

  return self
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === 'object' || typeof call === 'function')) {
    return call
  }

  return _assertThisInitialized(self)
}

function _createSuper(Derived) {
  return function () {
    var Super = _getPrototypeOf(Derived),
      result

    if (_isNativeReflectConstruct()) {
      var NewTarget = _getPrototypeOf(this).constructor

      result = Reflect.construct(Super, arguments, NewTarget)
    } else {
      result = Super.apply(this, arguments)
    }

    return _possibleConstructorReturn(this, result)
  }
}

var Config = {
  LIB_VERSION: '3.0.0',
  LIB_NAME: 'uniapp',
}

var _ = {}
var ArrayProto = Array.prototype,
  ObjProto = Object.prototype,
  slice = ArrayProto.slice,
  nativeToString = ObjProto.toString,
  nativeHasOwnProperty = Object.prototype.hasOwnProperty,
  nativeForEach = ArrayProto.forEach,
  nativeIsArray = Array.isArray,
  breaker = {}
var utmTypes = ['utm_source', 'utm_medium', 'utm_campaign', 'utm_content', 'utm_term']

_.each = function (obj, iterator, context) {
  // eslint-disable-next-line
  if (obj === null || obj === undefined) {
    return false
  }

  if (nativeForEach && obj.forEach === nativeForEach) {
    obj.forEach(iterator, context)
  } else if (obj.length === +obj.length) {
    for (var i = 0, l = obj.length; i < l; i++) {
      if (i in obj && iterator.call(context, obj[i], i, obj) === breaker) {
        return false
      }
    }
  } else {
    for (var key in obj) {
      if (nativeHasOwnProperty.call(obj, key)) {
        if (iterator.call(context, obj[key], key, obj) === breaker) {
          return false
        }
      }
    }
  }
}

_.extend = function (obj) {
  _.each(slice.call(arguments, 1), function (source) {
    for (var prop in source) {
      if (source[prop] !== void 0) {
        obj[prop] = source[prop]
      }
    }
  })

  return obj
}

_.extend2Layers = function (obj) {
  _.each(slice.call(arguments, 1), function (source) {
    for (var prop in source) {
      if (source[prop] !== void 0) {
        if (_.isObject(source[prop]) && _.isObject(obj[prop])) {
          _.extend(obj[prop], source[prop])
        } else {
          obj[prop] = source[prop]
        }
      }
    }
  })

  return obj
}

_.isArray =
  nativeIsArray ||
  function (obj) {
    return nativeToString.call(obj) === '[object Array]'
  }

_.isFunction = function (f) {
  try {
    return typeof f === 'function'
  } catch (x) {
    return false
  }
} //alipay request type

_.isPromise = function (obj) {
  return nativeToString.call(obj) === '[object Promise]' && obj !== null && obj !== undefined
}

_.isObject = function (obj) {
  return nativeToString.call(obj) === '[object Object]' && obj !== null && obj !== undefined
}

_.isEmptyObject = function (obj) {
  if (_.isObject(obj)) {
    for (var key in obj) {
      if (nativeHasOwnProperty.call(obj, key)) {
        return false
      }
    }

    return true
  }

  return false
}

_.isUndefined = function (obj) {
  return obj === void 0
}

_.isString = function (obj) {
  return nativeToString.call(obj) === '[object String]'
}

_.isDate = function (obj) {
  return nativeToString.call(obj) === '[object Date]'
}

_.isBoolean = function (obj) {
  return nativeToString.call(obj) === '[object Boolean]'
}

_.isNumber = function (obj) {
  // eslint-disable-next-line no-useless-escape
  return nativeToString.call(obj) === '[object Number]' && /[\d\.]+/.test(String(obj))
}

_.isJSONString = function (str) {
  try {
    JSON.parse(str)
  } catch (e) {
    return false
  }

  return true
}

_.decodeURIComponent = function (val) {
  var result = ''

  try {
    result = decodeURIComponent(val)
  } catch (e) {
    result = val
  }

  return result
}

_.encodeURIComponent = function (val) {
  var result = ''

  try {
    result = encodeURIComponent(val)
  } catch (e) {
    result = val
  }

  return result
}

_.utf8Encode = function (string) {
  string = (string + '').replace(/\r\n/g, '\n').replace(/\r/g, '\n')
  var utftext = ''
  var start, end
  var stringl = 0
  var n
  start = end = 0
  stringl = string.length

  for (n = 0; n < stringl; n++) {
    var c1 = string.charCodeAt(n)
    var enc = null

    if (c1 < 128) {
      end++
    } else if (c1 > 127 && c1 < 2048) {
      enc = String.fromCharCode((c1 >> 6) | 192, (c1 & 63) | 128)
    } else {
      enc = String.fromCharCode((c1 >> 12) | 224, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128)
    }

    if (enc !== null) {
      if (end > start) {
        utftext += string.substring(start, end)
      }

      utftext += enc
      start = end = n + 1
    }
  }

  if (end > start) {
    utftext += string.substring(start, string.length)
  }

  return utftext
}

_.base64Encode = function (data) {
  var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/='
  var o1, o2, o3, h1, h2, h3, h4, bits
  var i = 0,
    ac = 0,
    enc = '',
    tmpArr = []

  if (!data) {
    return data
  }

  data = _.utf8Encode(data)

  do {
    o1 = data.charCodeAt(i++)
    o2 = data.charCodeAt(i++)
    o3 = data.charCodeAt(i++)
    bits = (o1 << 16) | (o2 << 8) | o3
    h1 = (bits >> 18) & 0x3f
    h2 = (bits >> 12) & 0x3f
    h3 = (bits >> 6) & 0x3f
    h4 = bits & 0x3f
    tmpArr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4)
  } while (i < data.length)

  enc = tmpArr.join('')

  switch (data.length % 3) {
    case 1:
      enc = enc.slice(0, -2) + '=='
      break

    case 2:
      enc = enc.slice(0, -1) + '='
      break
  }

  return enc
}

_.encodeDates = function (obj) {
  _.each(obj, function (v, k) {
    if (_.isDate(v)) {
      obj[k] = _.formatDate(v)
    } else if (_.isObject(v)) {
      obj[k] = _.encodeDates(v)
    } else if (_.isArray(v)) {
      for (var i = 0; i < v.length; i++) {
        if (_.isDate(v[i])) {
          obj[k][i] = _.formatDate(v[i])
        }
      }
    }
  })

  return obj
}

_.formatDate = function (d) {
  function pad(n) {
    return n < 10 ? '0' + n : n
  }

  function secondPad(n) {
    if (n < 100 && n > 9) return '0' + n
    else if (n < 10) return '00' + n
    else return n
  }

  return d.getFullYear() + '-' + pad(d.getMonth() + 1) + '-' + pad(d.getDate()) + ' ' + pad(d.getHours()) + ':' + pad(d.getMinutes()) + ':' + pad(d.getSeconds()) + '.' + secondPad(d.getMilliseconds())
}

_.searchObjDate = function (o) {
  try {
    if (_.isObject(o) || _.isArray(o)) {
      _.each(o, function (a, b) {
        if (_.isObject(a) || _.isArray(a)) {
          _.searchObjDate(o[b])
        } else {
          if (_.isDate(a)) {
            o[b] = _.formatDate(a)
          }
        }
      })
    }
  } catch (err) {
    logger$1.warn(err)
  }
}

_.UUID = function () {
  var visitTime = new Date().getTime()
  var uuid = '' + String(Math.random()).replace('.', '').slice(1, 11) + '-' + visitTime
  return uuid
}

_.UUIDv4 = function () {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    var r = (Math.random() * 16) | 0,
      // eslint-disable-next-line eqeqeq
      v = c === 'x' ? r : (r & 0x3) | 0x8
    return v.toString(16)
  })
}

_.setMpPlatform = function (mpPlatform) {
  _.mpPlatform = mpPlatform
}

_.getMpPlatform = function () {
  return _.mpPlatform
}

_.createExtraHeaders = function () {
  return {
    'TA-Integration-Type': Config.LIB_NAME,
    'TA-Integration-Version': Config.LIB_VERSION,
    'TA-Integration-Count': '1',
    'TA-Integration-Extra': _.getMpPlatform(),
  }
} // remove spaces in AppId

_.checkAppId = function (appId) {
  appId = appId.replace(/\s*/g, '')
  return appId
} // remove spaces, pathname (file name), other parameters in URL

_.checkUrl = function (url) {
  url = url.replace(/\s*/g, '')
  url = _.url('basic', url)
  return url
}

_.url = (function () {
  function _t() {
    return new RegExp(
      /(.*?)\.?([^.]*?)\.(com|net|org|biz|ws|in|me|co\.uk|co|org\.uk|ltd\.uk|plc\.uk|me\.uk|edu|mil|br\.com|cn\.com|eu\.com|hu\.com|no\.com|qc\.com|sa\.com|se\.com|se\.net|us\.com|uy\.com|ac|co\.ac|gv\.ac|or\.ac|ac\.ac|af|am|as|at|ac\.at|co\.at|gv\.at|or\.at|asn\.au|com\.au|edu\.au|org\.au|net\.au|id\.au|be|ac\.be|adm\.br|adv\.br|am\.br|arq\.br|art\.br|bio\.br|cng\.br|cnt\.br|com\.br|ecn\.br|eng\.br|esp\.br|etc\.br|eti\.br|fm\.br|fot\.br|fst\.br|g12\.br|gov\.br|ind\.br|inf\.br|jor\.br|lel\.br|med\.br|mil\.br|net\.br|nom\.br|ntr\.br|odo\.br|org\.br|ppg\.br|pro\.br|psc\.br|psi\.br|rec\.br|slg\.br|tmp\.br|tur\.br|tv\.br|vet\.br|zlg\.br|br|ab\.ca|bc\.ca|mb\.ca|nb\.ca|nf\.ca|ns\.ca|nt\.ca|on\.ca|pe\.ca|qc\.ca|sk\.ca|yk\.ca|ca|cc|ac\.cn|net\.cn|com\.cn|edu\.cn|gov\.cn|org\.cn|bj\.cn|sh\.cn|tj\.cn|cq\.cn|he\.cn|nm\.cn|ln\.cn|jl\.cn|hl\.cn|js\.cn|zj\.cn|ah\.cn|gd\.cn|gx\.cn|hi\.cn|sc\.cn|gz\.cn|yn\.cn|xz\.cn|sn\.cn|gs\.cn|qh\.cn|nx\.cn|xj\.cn|tw\.cn|hk\.cn|mo\.cn|cn|cx|cz|de|dk|fo|com\.ec|tm\.fr|com\.fr|asso\.fr|presse\.fr|fr|gf|gs|co\.il|net\.il|ac\.il|k12\.il|gov\.il|muni\.il|ac\.in|co\.in|org\.in|ernet\.in|gov\.in|net\.in|res\.in|is|it|ac\.jp|co\.jp|go\.jp|or\.jp|ne\.jp|ac\.kr|co\.kr|go\.kr|ne\.kr|nm\.kr|or\.kr|li|lt|lu|asso\.mc|tm\.mc|com\.mm|org\.mm|net\.mm|edu\.mm|gov\.mm|ms|nl|no|nu|pl|ro|org\.ro|store\.ro|tm\.ro|firm\.ro|www\.ro|arts\.ro|rec\.ro|info\.ro|nom\.ro|nt\.ro|se|si|com\.sg|org\.sg|net\.sg|gov\.sg|sk|st|tf|ac\.th|co\.th|go\.th|mi\.th|net\.th|or\.th|tm|to|com\.tr|edu\.tr|gov\.tr|k12\.tr|net\.tr|org\.tr|com\.tw|org\.tw|net\.tw|ac\.uk|uk\.com|uk\.net|gb\.com|gb\.net|vg|sh|kz|ch|info|ua|gov|name|pro|ie|hk|com\.hk|org\.hk|net\.hk|edu\.hk|us|tk|cd|by|ad|lv|eu\.lv|bz|es|jp|cl|ag|mobi|eu|co\.nz|org\.nz|net\.nz|maori\.nz|iwi\.nz|io|la|md|sc|sg|vc|tw|travel|my|se|tv|pt|com\.pt|edu\.pt|asia|fi|com\.ve|net\.ve|fi|org\.ve|web\.ve|info\.ve|co\.ve|tel|im|gr|ru|net\.ru|org\.ru|hr|com\.hr|ly|xyz)$/
    )
  }

  function _d(s) {
    return _.decodeURIComponent(s.replace(/\+/g, ' '))
  }

  function _i(arg, str) {
    var sptr = arg.charAt(0)
    var split = str.split(sptr)

    if (sptr === arg) {
      return split
    }

    arg = parseInt(arg.substring(1), 10)
    return split[arg < 0 ? split.length + arg : arg - 1]
  }

  function _f(arg, str) {
    var sptr = arg.charAt(0)
    var split = str.split('&')
    var field = []
    var params = {}
    var tmp = []
    var arg2 = arg.substring(1)

    for (var i = 0, ii = split.length; i < ii; i++) {
      field = split[i].match(/(.*?)=(.*)/) // TODO: regex should be able to handle this.

      if (!field) {
        field = [split[i], split[i], '']
      }

      if (field[1].replace(/\s/g, '') !== '') {
        field[2] = _d(field[2] || '') // If we have a match just return it right away.

        if (arg2 === field[1]) {
          return field[2]
        } // Check for array pattern.

        tmp = field[1].match(/(.*)\[([0-9]+)\]/)

        if (tmp) {
          params[tmp[1]] = params[tmp[1]] || []
          params[tmp[1]][tmp[2]] = field[2]
        } else {
          params[field[1]] = field[2]
        }
      }
    }

    if (sptr === arg) {
      return params
    }

    return params[arg2]
  }

  return function (arg, url) {
    var _l = {},
      tmp

    if (arg === 'tld?') {
      return _t()
    }

    url = url || window.location.toString()

    if (!arg) {
      return url
    }

    arg = arg.toString()

    if (url.match(/^mailto:([^/].+)/)) {
      tmp = url.match(/^mailto:([^/].+)/)
      _l.protocol = 'mailto'
      _l.email = tmp[1]
    } else {
      // Ignore Hashbangs.
      if (url.match(/(.*?)\/#!(.*)/)) {
        tmp = url.match(/(.*?)\/#!(.*)/)
        url = tmp[1] + tmp[2]
      } // Hash.

      if (url.match(/(.*?)#(.*)/)) {
        tmp = url.match(/(.*?)#(.*)/)
        _l.hash = tmp[2]
        url = tmp[1]
      } // Return hash parts.

      if (_l.hash && arg.match(/^#/)) {
        return _f(arg, _l.hash)
      } // Query

      if (url.match(/(.*?)\?(.*)/)) {
        tmp = url.match(/(.*?)\?(.*)/)
        _l.query = tmp[2]
        url = tmp[1]
      } // Return query parts.

      if (_l.query && arg.match(/^\?/)) {
        return _f(arg, _l.query)
      } // Protocol.

      if (url.match(/(.*?):?\/\/(.*)/)) {
        tmp = url.match(/(.*?):?\/\/(.*)/)
        _l.protocol = tmp[1].toLowerCase()
        url = tmp[2]
      } // Path.

      if (url.match(/(.*?)(\/.*)/)) {
        tmp = url.match(/(.*?)(\/.*)/)
        _l.path = tmp[2]
        url = tmp[1]
      } // Clean up path.

      _l.path = (_l.path || '').replace(/^([^/])/, '/$1').replace(/\/$/, '') // Return path parts.

      if (arg.match(/^[-0-9]+$/)) {
        arg = arg.replace(/^([^/])/, '/$1')
      }

      if (arg.match(/^\//)) {
        return _i(arg, _l.path.substring(1))
      } // File.

      tmp = _i('/-1', _l.path.substring(1))

      if (tmp && (tmp = tmp.match(/(.*?)\.(.*)/))) {
        _l.file = tmp[0]
        _l.filename = tmp[1]
        _l.fileext = tmp[2]
      } // Port.

      if (url.match(/(.*):([0-9]+)$/)) {
        tmp = url.match(/(.*):([0-9]+)$/)
        _l.port = tmp[2]
        url = tmp[1]
      } // Auth.

      if (url.match(/(.*?)@(.*)/)) {
        tmp = url.match(/(.*?)@(.*)/)
        _l.auth = tmp[1]
        url = tmp[2]
      } // User and pass.

      if (_l.auth) {
        tmp = _l.auth.match(/(.*):(.*)/)
        _l.user = tmp ? tmp[1] : _l.auth
        _l.pass = tmp ? tmp[2] : undefined
      } // Hostname.

      _l.hostname = url.toLowerCase() // Return hostname parts.

      if (arg.charAt(0) === '.') {
        return _i(arg, _l.hostname)
      } // Domain, tld and sub domain.

      if (_t()) {
        tmp = _l.hostname.match(_t())

        if (tmp) {
          _l.tld = tmp[3]
          _l.domain = tmp[2] ? tmp[2] + '.' + tmp[3] : undefined
          _l.sub = tmp[1] || undefined
        }
      } // Set port and protocol defaults if not set.

      var portInfo = _l.port ? ':' + _l.port : ''
      _l.protocol = _l.protocol || window.location.protocol.replace(':', '') // console.log(_l);

      _l.port = _l.port || (_l.protocol === 'https' ? '443' : '80')
      _l.protocol = _l.protocol || (_l.port === '443' ? 'https' : 'http')
      _l.basic = _l.protocol + '://' + _l.hostname + portInfo
    } // Return arg.

    if (arg in _l) {
      return _l[arg]
    } // Return everything.

    if (arg === '{}') {
      return _l
    } // Default to undefined for no match.

    return ''
  }
})()

_.createString = function (length) {
  var expect = length
  var str = Math.random().toString(36).substr(2)

  while (str.length < expect) {
    str += Math.random().toString(36).substr(2)
  }

  str = str.substr(0, length)
  return str
}

_.createAesKey = function () {
  return _.createString(16)
}

_.generateEncryptyData = function (text, secretKey) {
  if (typeof secretKey === 'undefined') {
    return text
  }

  var pkey = secretKey['publicKey']
  var v = secretKey['version']

  if (typeof pkey === 'undefined' || typeof v === 'undefined') {
    return text
  }

  if (typeof CryptoJS === 'undefined' || typeof JSEncrypt === 'undefined') {
    return text
  }

  var strKey = _.createAesKey()

  try {
    var key = CryptoJS.enc.Utf8.parse(strKey)
    var data = CryptoJS.enc.Utf8.parse(JSON.stringify(text))
    var padding = _.isUndefined(CryptoJS.pad.Pkcs7) ? CryptoJS.pad.PKCS7 : CryptoJS.pad.Pkcs7
    var aesStr = CryptoJS.AES.encrypt(data, key, {
      mode: CryptoJS.mode.ECB,
      padding: padding,
    }).toString()
    var encrypt = new JSEncrypt()
    encrypt.setPublicKey(pkey)
    var rsaStr = encrypt.encrypt(strKey)

    if (rsaStr === false) {
      logger$1.warn('Encryption failed, return the original data')
      return text
    }

    return {
      pkv: v,
      ekey: rsaStr,
      payload: aesStr,
    }
  } catch (e) {
    logger$1.warn('Encryption failed, return the original data: ' + e)
  }

  return text
}

_.getUtm = function () {
  var params = {}

  _.each(utmTypes, function (kwkey) {
    try {
      var kw = _.getQueryParam(location.href, kwkey)

      if (kw.length) {
        params[kwkey] = kw
      }
    } catch (e) {
      logger$1.warn('get utm fail: ' + e)
    }
  })

  return JSON.stringify(params)
}

_.getQueryParam = function (url, key) {
  key = key.replace(/[\[]/, '\\[').replace(/[\]]/, '\\]')
  url = _.decodeURIComponent(url)
  var regexS = '[\\?&]' + key + '=([^&#]*)',
    regex = new RegExp(regexS),
    results = regex.exec(url)

  if (results === null || (results && typeof results[1] !== 'string' && results[1].length)) {
    return ''
  } else {
    return _.decodeURIComponent(results[1])
  }
}

_.getUtmFromQuery = function (query) {
  var params = {}

  _.each(utmTypes, function (kwkey) {
    if (query[kwkey]) {
      params[kwkey] = query[kwkey]
    }
  })

  return JSON.stringify(params)
}

_.indexOf = function (arr, target) {
  var indexof = arr.indexOf

  if (indexof) {
    return indexof.call(arr, target)
  } else {
    for (var i = 0; i < arr.length; i++) {
      if (target === arr[i]) {
        return i
      }
    }

    return -1
  }
}

_.checkCalibration = function (properties, time, enableCalibrationTime) {
  // if(!enableCalibrationTime){
  //     return properties;
  // }
  // if (properties && properties['#time_calibration']) {
  //     return;
  // }
  // var pro = {};
  // var timeCalibration = 6;
  // if (enableCalibrationTime) {
  //     if (_.isDate(time)) {
  //         timeCalibration = 5;
  //     } else {
  //         timeCalibration = 3;
  //     }
  // }
  // return _.extend(pro,properties,{'#time_calibration':timeCalibration});
  return properties
}

var logger$1 = _typeof(logger$1) === 'object' ? logger$1 : {}

logger$1.info = function () {
  if ((typeof console === 'undefined' ? 'undefined' : _typeof(console)) === 'object' && console.log && logger$1.enabled) {
    try {
      arguments[0] = '[ThinkingData] Info: ' + arguments[0]
      return console.log.apply(console, arguments)
    } catch (e) {
      console.log('[ThinkingData] Info: ' + arguments[0])
    }
  }
}

logger$1.warn = function () {
  if ((typeof console === 'undefined' ? 'undefined' : _typeof(console)) === 'object' && console.log && logger$1.enabled) {
    try {
      arguments[0] = '[ThinkingData] Warning: ' + arguments[0]
      return console.warn.apply(console, arguments)
    } catch (e) {
      console.warn('[ThinkingData] Warning: ' + arguments[0])
    }
  }
}

var PlatformProxy = /*#__PURE__*/ (function () {
  function PlatformProxy() {
    _classCallCheck(this, PlatformProxy)

    this.config = {
      persistenceName: 'thinkingdata',
      persistenceNameOld: 'thinkingdata_mg',
    }
  }

  _createClass(
    PlatformProxy,
    [
      {
        key: 'getConfig',

        /**
         * Get platform specific configuration: persistenceName required
         */
        value: function getConfig() {
          return this.config
        },
        /**
         * Get local cache data
         * @param {string} name: cache key
         * @param {boolean} async: enable asynchronous getting cached
         * @param {function} callback: callback when getting data asynchronously, the parameter is an object
         * @return return cached data, it is an object
         */
      },
      {
        key: 'getStorage',
        value: function getStorage(name, async, callback) {
          // if (async) logger.warn('TA: invalid storage configuration');
          var data = localStorage.getItem(name)

          if (async) {
            if (_.isJSONString(data)) {
              callback(JSON.parse(data))
            } else {
              callback({})
            }
          } else {
            if (_.isJSONString(data)) {
              return JSON.parse(data)
            } else {
              return {}
            }
          }
        },
        /**
         * Set local cache data
         * @param {string} name: cache key
         * @param {string} value: JSON string value
         */
      },
      {
        key: 'setStorage',
        value: function setStorage(name, value) {
          localStorage.setItem(name, value)
        },
        /**
         * Delete data in local cache with key
         * @param {*} name: cache key
         */
      },
      {
        key: 'removeStorage',
        value: function removeStorage(name) {
          localStorage.removeItem(name)
        },
      },
      {
        key: '_setSystemProxy',
        value: function _setSystemProxy(callback) {
          this._sysCallback = callback
        },
        /**
         * Get system information asynchronously
         * @param {object} options: callback when getting completion
         * callback parameter：
         * brand: string, device brand
         * model: string, device model
         * screenWidth: number, screen width, unit px
         * screenHeight: number, screen height, unit px
         * system: string, operating system and version
         * platform: string, client platform
         */
      },
      {
        key: 'getSystemInfo',
        value: function getSystemInfo(options) {
          var res = {
            // eslint-disable-next-line
            mp_platform: 'web',
            system: this._getOs(),
            screenWidth: window.screen.width,
            screenHeight: window.screen.height,
            systemLanguage: navigator.language,
          }

          if (this._sysCallback) {
            res = _.extend(res, this._sysCallback(options))
          }

          options.success(res)
          options.complete()
        },
      },
      {
        key: '_getOs',
        value: function _getOs() {
          var a = navigator.userAgent

          if (/Windows/i.test(a)) {
            if (/Phone/.test(a) || /WPDesktop/.test(a)) {
              return 'Windows Phone'
            }

            return 'Windows'
          } else if (/(iPhone|iPad|iPod)/.test(a)) {
            return 'iOS'
          } else if (/Android/.test(a)) {
            return 'Android'
          } else if (/(BlackBerry|PlayBook|BB10)/i.test(a)) {
            return 'BlackBerry'
          } else if (/Mac/i.test(a)) {
            return 'MacOS'
          } else if (/Linux/.test(a)) {
            return 'Linux'
          } else if (/CrOS/.test(a)) {
            return 'ChromeOS'
          } else {
            return ''
          }
        },
        /**
         * Get network type asynchronously
         * @param {object} options: callback when getting completion
         * res.networkType string: network type
         */
      },
      {
        key: 'getNetworkType',
        value: function getNetworkType(options) {
          options.complete()
        },
        /**
         * Listen for network state change
         * @param {function} callback: callback when network state changing
         */
      },
      {
        key: 'onNetworkStatusChange',
        value: function onNetworkStatusChange() {},
        /**
         * Make a network request
         * @param {object} options: parameters, including:
         *   url       string         server url
         *   data      string/object  request parameters
         *   method    string         HTTP method
         *   success   function       success callback
         *   fail      function       fail callback
         *   complete  function       complete callback
         */
      },
      {
        key: 'request',
        value: function request(options) {
          var res = {}
          var xhr = new XMLHttpRequest()
          xhr.open(options.method, options.url)

          if (options.header) {
            for (var key in options.header) {
              xhr.setRequestHeader(key, options.header[key])
            }
          }

          xhr.onreadystatechange = function () {
            if (xhr.readyState === 4 && xhr.status === 200) {
              res['statusCode'] = 200

              if (_.isJSONString(xhr.responseText)) {
                res['data'] = JSON.parse(xhr.responseText)
              }

              options.success(res)
            } else if (xhr.status !== 200) {
              res.errMsg = 'network error'
              options.fail(res)
            }
          }

          xhr.ontimeout = function () {
            res.errMsg = 'timeout'
            options.fail(res)
          }

          xhr.send(options.data)
          return xhr
        },
      },
      {
        key: 'initAutoTrackInstance',
        value: function initAutoTrackInstance(instance, config) {
          this.instance = instance
          this.autoTrack = config.autoTrack

          var _that = this

          _that.onPageShow()

          if (_that.autoTrack.appHide) {
            _that.instance.timeEvent('ta_page_hide')
          }

          if ('onvisibilitychange' in document) {
            document.onvisibilitychange = function () {
              if (document.hidden) {
                _that.onPageHide(true)
              } else {
                _that.onPageShow()

                if (_that.autoTrack.appHide) {
                  _that.instance.timeEvent('ta_page_hide')
                }
              }
            }
          }
        },
      },
      {
        key: 'setGlobal',
        value: function setGlobal(instance, name) {
          window[name] = instance
        },
        /**
         * Get system startup information, and register APP cut-off foreground callback
         * @TODO
         */
      },
      {
        key: 'getAppOptions',
        value: function getAppOptions() {},
        /**
         * Toast Debug information
         * @param {string} msg: information to display
         */
      },
      {
        key: 'showToast',
        value: function showToast() {},
      },
      {
        key: 'onPageShow',
        value: function onPageShow() {
          if (this.autoTrack.appShow) {
            var properties = {}

            _.extend(properties, this.autoTrack.properties)

            if (_.isFunction(this.autoTrack.callback)) {
              _.extend(properties, this.autoTrack.callback('appShow'))
            }

            this.instance._internalTrack('ta_page_show', properties)
          }
        },
      },
      {
        key: 'onPageHide',
        value: function onPageHide(tryBeacon) {
          if (this.autoTrack.appHide) {
            var properties = {}

            _.extend(properties, this.autoTrack.properties)

            if (_.isFunction(this.autoTrack.callback)) {
              _.extend(properties, this.autoTrack.callback('appHide'))
            }

            this.instance._internalTrack('ta_page_hide', properties, new Date(), null, tryBeacon)
          }
        },
      },
    ],
    [
      {
        key: 'createInstance',
        value: function createInstance() {
          return new PlatformProxy()
        },
      },
    ]
  )

  return PlatformProxy
})()

var AutoTrackBridge = /*#__PURE__*/ (function () {
  function AutoTrackBridge(instance, config) {
    _classCallCheck(this, AutoTrackBridge)

    this.taInstance = instance
    this.config = config || {}
    this.referrer = 'Directly open'

    if (this.config.isPlugin) {
      instance.App = function () {
        App.apply(this, arguments)
      }

      inension(instance.Page)
    } else {
      var originalApp = App
      App = this._initAppExtention(originalApp)
      var originalPage = Page
      Page = this._initPageExtension(originalPage)
    }
  }

  _createClass(AutoTrackBridge, [
    {
      key: '_initPageExtension',
      value: function _initPageExtension(Page) {
        var _that = this

        return function (page) {
          var onShow = page.onShow,
            onShareAppMessage = page.onShareAppMessage

          page.onShow = function (options) {
            _that.onPageShow()

            if (typeof onShow === 'function') {
              onShow.call(this, options)
            }
          }

          if (typeof onShareAppMessage === 'function') {
            page.onShareAppMessage = function (object) {
              var ret = onShareAppMessage.call(this, object)
              return _that.onPageShare(ret)
            }
          }

          return Page(page)
        }
      },
    },
    {
      key: '_initAppExtention',
      value: function _initAppExtention(App) {
        var _that = this

        return function (app) {
          var onLaunch = app.onLaunch,
            onShow = app.onShow,
            onHide = app.onHide

          app.onLaunch = function (options) {
            _that.onAppLaunch(options, this)

            if (typeof onLaunch === 'function') {
              onLaunch.call(this, options)
            }
          }

          app.onShow = function (options) {
            _that.onAppShow(options)

            if (typeof onShow === 'function') {
              onShow.call(this, options)
            }
          }

          app.onHide = function () {
            _that.onAppHide()

            if (typeof onHide === 'function') {
              onHide.call(this)
            }
          }

          return App(app)
        }
      },
    },
    {
      key: 'onAppLaunch',
      value: function onAppLaunch(options, app) {
        this._setAutoTrackProperties(options)

        if (!_.isUndefined(app)) {
          app[this.taInstance.name] = this.taInstance
        }

        if (this.config.appLaunch) {
          var prop = {}

          if (options && options.path) {
            prop['#url_path'] = this._getPath(options.path)
          }

          if (options && options.query) {
            prop['#utm'] = _.getUtmFromQuery(options.query)
          }

          this.taInstance._internalTrack('ta_mp_launch', prop)
        }
      },
    },
    {
      key: 'onAppShow',
      value: function onAppShow(options) {
        if (this.config.appHide) {
          this.taInstance.timeEvent('ta_mp_hide')
        }

        this._setAutoTrackProperties(options)

        if (this.config.appShow) {
          var prop = {}

          if (options && options.path) {
            prop['#url_path'] = this._getPath(options.path)
          }

          if (options && options.query) {
            prop['#utm'] = _.getUtmFromQuery(options.query)
          }

          _.extend(prop, this.config.properties)

          if (_.isFunction(this.config.callback)) {
            _.extend(prop, this.config.callback('appShow'))
          }

          this.taInstance._internalTrack('ta_mp_show', prop)
        }
      },
    },
    {
      key: 'onAppHide',
      value: function onAppHide() {
        if (this.config.appHide) {
          var prop = {
            '#url_path': this._getCurrentPath(),
          }

          _.extend(prop, this.config.properties)

          if (_.isFunction(this.config.callback)) {
            _.extend(prop, this.config.callback('appHide'))
          }

          this.taInstance._internalTrack('ta_mp_hide', prop)
        }
      },
    },
    {
      key: '_getCurrentPath',
      value: function _getCurrentPath() {
        var url = 'Not to get'

        try {
          // eslint-disable-next-line no-undef
          var pages = getCurrentPages()
          var currentPage = pages[pages.length - 1]
          url = currentPage.route // Modify carefully, the ByteDance applet needs to replace this line of code. If you need to modify, please modify rollup.config.js synchronously
        } catch (e) {
          logger$1.info(e)
        }

        return url
      },
    },
    {
      key: '_setAutoTrackProperties',
      value: function _setAutoTrackProperties(options) {
        var props = {
          '#scene': options.scene,
        }
        /*
      if (options && _.isObject(options.query) && options.query.tashare) {
          var shareInfo = _.decodeURIComponent(options.query.tashare);
          if (_.isJSONString(shareInfo)) {
              this.shareInfo = JSON.parse(shareInfo);
              props['#share_depth'] = _.isNumber(this.shareInfo.d) ? this.shareInfo.d : DEFAULT_SHARE_DEPTH;
          }
      }
      */

        this.taInstance._setAutoTrackProperties(props)
      },
      /*
    _getShareDepth() {
        var shareInfo = this.shareInfo || {};
        if ((shareInfo.a && shareInfo.a === this.taInstance.getAccountId()) || (shareInfo.i && shareInfo.i === this.taInstance.getDistinctId())) {
            return shareInfo.d;
        } else if (shareInfo.d) {
            return shareInfo.d + 1;
        } else {
            return DEFAULT_SHARE_DEPTH;
        }
    }
    */
    },
    {
      key: '_getPath',
      value: function _getPath(path) {
        return (path = 'string' === typeof path ? path.replace(/^\//, '') : 'Abnormal values')
      },
      /*
    _generateShareInfo() {
         return JSON.stringify({
            i: this.taInstance.getDistinctId(),
            a: this.taInstance.getAccountId(),
            p: this._getCurrentPath(),
            d: this._getShareDepth(),
        });
    }*/
    },
    {
      key: 'onPageShare',
      value: function onPageShare(result) {
        if (this.config.pageShare) {
          this.taInstance._internalTrack('ta_mp_share', {
            '#url_path': this._getCurrentPath(),
          })
        }

        var ret = _.isObject(result) ? result : {}
        /*
      if (this.config.allow_share_info) {
          if (_.isUndefined(ret.path) || ret.path === '') {
              ret.path = this._getCurrentPath();
          }
           if (_.isString(ret.path)) {
              if (-1 === ret.path.indexOf('?')) {
                  ret.path = ret.path + '?';
              } else if ('&' !== ret.path.slice(-1)) {
                  ret.path = ret.path + '&';
              }
              ret.path = ret.path + 'tashare=' + encodeURIComponent(this._generateShareInfo());
          }
      }*/

        return ret
      },
    },
    {
      key: 'onPageShow',
      value: function onPageShow() {
        if (this.config.pageShow) {
          var path = this._getCurrentPath()

          var prop = {
            '#url_path': path || 'The system did not get a value',
            '#referrer': this.referrer,
          }
          this.referrer = path

          this.taInstance._internalTrack('ta_mp_view', prop)
        }
      },
    },
  ])

  return AutoTrackBridge
})()

var AutoTrackBridge$1 = /*#__PURE__*/ (function () {
  function AutoTrackBridge(instance, config, currentPlatform) {
    var _this = this

    _classCallCheck(this, AutoTrackBridge)

    this.taInstance = instance
    this.config = config || {}
    var options = currentPlatform.getLaunchOptionsSync()

    this._onShow(options)

    this.startTracked = true
    currentPlatform.onShow(function (options) {
      _this._onShow(options)
    })
    currentPlatform.onHide(function () {
      _this.startTracked = false

      if (_this.config.appHide) {
        var properties = {}

        _.extend(properties, _this.config.properties)

        if (_.isFunction(_this.config.callback)) {
          _.extend(properties, _this.config.callback('appHide'))
        }

        _this.taInstance._internalTrack('ta_mg_hide', properties)
      }
    })
  }

  _createClass(AutoTrackBridge, [
    {
      key: '_onShow',
      value: function _onShow(options) {
        if (this.startTracked) return

        if (this.config.appHide) {
          this.taInstance.timeEvent('ta_mg_hide')
        }

        if (options && options.scene) {
          this.taInstance._setAutoTrackProperties({
            '#scene': options.scene,
          })
        }

        if (this.config.appShow) {
          var properties = {}

          _.extend(properties, this.config.properties)

          if (_.isFunction(this.config.callback)) {
            _.extend(properties, this.config.callback('appShow'))
          }

          this.taInstance._internalTrack('ta_mg_show', properties)
        }
      },
    },
  ])

  return AutoTrackBridge
})()

var PlatformProxy$1 = /*#__PURE__*/ (function () {
  function PlatformProxy$1(api, platformConfig, internalConfig) {
    _classCallCheck(this, PlatformProxy$1)

    this.api = api
    this.config = platformConfig
    this._config = internalConfig
  }

  _createClass(
    PlatformProxy$1,
    [
      {
        key: 'getConfig',

        /**
         * Get platform specific configuration: persistenceName required
         */
        value: function getConfig() {
          return this.config
        },
        /**
         * Get local cache data
         * @param {string} name: cache key
         * @param {boolean} async: enable asynchronous getting cached
         * @param {function} callback: callback when getting data asynchronously, the parameter is an object
         * @return return cached data, it is an object
         */
      },
      {
        key: 'getStorage',
        value: function getStorage(name, async, callback) {
          if (async) {
            this.api.getStorage({
              key: name,
              success: function success(res) {
                var data = _.isJSONString(res.data) ? JSON.parse(res.data) : {}
                callback(data)
              },
              fail: function fail() {
                logger$1.warn('getStorage faild')
                callback({})
              },
            })
          } else {
            if (this._config.platform === 'dd_mp') {
              var res = this.api.getStorageSync({
                key: name,
              })

              if (_.isJSONString(res.data)) {
                return JSON.parse(res.data)
              } else {
                return {}
              }
            }

            var data = this.api.getStorageSync(name)

            if (_.isJSONString(data)) {
              return JSON.parse(data)
            } else {
              return {}
            }
          }
        },
        /**
         * Set local cache data
         * @param {string} name: cache key
         * @param {string} value: JSON string value
         */
      },
      {
        key: 'setStorage',
        value: function setStorage(name, value) {
          if (this._config.platform === 'ali_mp' || this._config.platform === 'tb_mp' || this._config.platform === 'dd_mp') {
            this.api.setStorageSync({
              key: name,
              data: value,
            })
          } else {
            this.api.setStorageSync(name, value)
          }
        },
        /**
         * Delete data in local cache with key
         * @param {*} name: cache key
         */
      },
      {
        key: 'removeStorage',
        value: function removeStorage(name) {
          if (_.isFunction(this.api.removeStorage)) {
            this.api.removeStorage({
              key: name,
            })
          } else if (_.isFunction(this.api.deleteStorage)) {
            this.api.deleteStorage({
              key: name,
            })
          }
        },
      },
      {
        key: '_getPlatform',
        value: function _getPlatform() {
          return ''
        },
        /**
         * Get system information asynchronously
         * @param {object} options: callback when getting completion
         * callback parameter：
         * brand: string, device brand
         * model: string, device model
         * screenWidth: number, screen width, unit px
         * screenHeight: number, screen height, unit px
         * system: string, operating system and version
         * platform: string, client platform
         */
      },
      {
        key: 'getSystemInfo',
        value: function getSystemInfo(options) {
          var platform = this._config.mpPlatform
          this.api.getSystemInfo({
            success: function success(res) {
              if (_.isFunction(platform)) {
                res['mp_platform'] = platform(res)
              } else {
                res['mp_platform'] = platform
              }

              options.success(res)

              if (platform === 'wechat') {
                //Sometimes the WeChat platform complete will not call back,
                //you need to call options.complete in the success callback to complete the acquisition of system information
                options.complete()
              }
            },
            complete: function complete() {
              options.complete()
            },
          })
        },
        /**
         * Get network type asynchronously
         * @param {object} options: callback when getting completion
         * res.networkType string: network type
         */
      },
      {
        key: 'getNetworkType',
        value: function getNetworkType(options) {
          if (!_.isFunction(this.api.getNetworkType)) {
            options.success({})
            options.complete()
          } else {
            this.api.getNetworkType({
              success: function success(res) {
                options.success(res)
              },
              complete: function complete() {
                options.complete()
              },
            })
          }
        },
        /**
         * Listen for network state change
         * @param {function} callback: callback when network state changing
         */
      },
      {
        key: 'onNetworkStatusChange',
        value: function onNetworkStatusChange(callback) {
          if (!_.isFunction(this.api.onNetworkStatusChange)) {
            callback({})
          } else {
            this.api.onNetworkStatusChange(callback)
          }
        },
        /**
         * Make a network request
         * @param {object} options: parameters, including:
         *   url       string         server url
         *   data      string/object  request parameters
         *   method    string         HTTP method
         *   success   function       success callback
         *   fail      function       fail callback
         *   complete  function       complete callback
         */
      },
      {
        key: 'request',
        value: function request(options) {
          if (this._config.platform === 'ali_mp' || this._config.platform === 'dd_mp') {
            var config = _.extend({}, options)

            config.headers = options.header

            config.success = function (res) {
              res.statusCode = res.status
              options.success(res)
            }

            config.fail = function (res) {
              res.errMsg = res.errorMessage
              options.fail(res)
            }

            if (this._config.platform === 'dd_mp') {
              return this.api.httpRequest(config)
            } else {
              return this.api.request(config)
            }
          } else {
            return this.api.request(options)
          }
        },
        /**
         * Initialize the lifecycle monitoring instance
         * @param {ThinkingDataAPI} instance: SDK instance, listen lifecycle of application
         * @param {object} config: auto-tracking events config
         */
      },
      {
        key: 'initAutoTrackInstance',
        value: function initAutoTrackInstance(instance, config) {
          if (_.isObject(config.autoTrack)) {
            config.autoTrack['isPlugin'] = config.is_plugin
          }

          if (this._config.mp) {
            return new AutoTrackBridge(instance, config.autoTrack, this.api)
          } else {
            return new AutoTrackBridge$1(instance, config.autoTrack, this.api)
          }
        },
      },
      {
        key: 'setGlobal',
        value: function setGlobal(instance, name) {
          if (this._config.mp) {
            logger$1.warn('ThinkingAnalytics: we do not set global name for TA instance when you do not enable auto track.')
          } else {
            GameGlobal[name] = instance
          }
        },
        /**
         * Get system startup information, and register APP cut-off foreground callback
         * @param {function} callback
         */
      },
      {
        key: 'getAppOptions',
        value: function getAppOptions(callback) {
          var options = {}

          try {
            options = this.api.getLaunchOptionsSync()
          } catch (e) {
            logger$1.warn('Cannot get launch options.')
          }

          if (_.isFunction(callback)) {
            try {
              if (this._config.mp) {
                this.api.onAppShow(callback)
              } else {
                this.api.onShow(callback)
              }
            } catch (e) {
              logger$1.warn('Cannot register onShow callback.')
            }
          }

          return options
        },
        /**
         * Toast Debug information
         * @param {string} msg: information to display
         */
      },
      {
        key: 'showToast',
        value: function showToast(msg) {
          if (_.isFunction(this.api.showToast)) {
            var content = {
              title: msg,
            }

            if (this._config.platform === 'dd_mp' || this._config.platform === 'ali_mp') {
              content.content = msg
            }

            this.api.showToast(content)
          }
        },
      },
    ],
    [
      {
        key: 'createInstance',
        value: function createInstance() {
          // rollup will replace the following strings with the corresponding platforms when packaging
          return this._createInstance('R_CURRENT_PLATFORM')
        },
      },
      {
        key: '_createInstance',
        value: function _createInstance(option) {
          switch (option) {
            // for historical reason, we use different persistence names for different platforms.
            case 'wechat_mp':
              return new PlatformProxy$1(
                wx,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_wechat',
                },
                {
                  mpPlatform: 'wechat',
                  mp: true,
                  platform: option,
                }
              )

            case 'wechat_mg':
              return new PlatformProxy$1(
                wx,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_wechat_game',
                },
                {
                  mpPlatform: 'wechat',
                  platform: option,
                }
              )

            case 'qq_mp':
              return new PlatformProxy$1(
                qq,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_qq',
                },
                {
                  mpPlatform: 'qq',
                  mp: true,
                  platform: option,
                }
              )

            case 'qq_mg':
              return new PlatformProxy$1(
                qq,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_qq_game',
                },
                {
                  mpPlatform: 'qq',
                  platform: option,
                }
              )

            case 'baidu_mp':
              return new PlatformProxy$1(
                swan,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_swan',
                },
                {
                  mpPlatform: function mpPlatform(res) {
                    return res['host']
                  },
                  mp: true,
                  platform: option,
                }
              )

            case 'baidu_mg':
              return new PlatformProxy$1(
                swan,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_swan_game',
                },
                {
                  mpPlatform: function mpPlatform(res) {
                    return res['host']
                  },
                  platform: option,
                }
              )

            case 'tt_mg':
              return new PlatformProxy$1(
                tt,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_tt_game',
                },
                {
                  mpPlatform: function mpPlatform(res) {
                    return res['appName']
                  },
                  platform: option,
                }
              )

            case 'tt_mp':
              return new PlatformProxy$1(
                tt,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_tt',
                },
                {
                  mpPlatform: function mpPlatform(res) {
                    return res['appName']
                  },
                  mp: true,
                  platform: option,
                }
              )

            case 'ali_mp':
              return new PlatformProxy$1(
                my,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_ali',
                },
                {
                  mpPlatform: function mpPlatform(res) {
                    return res['app']
                  },
                  mp: true,
                  platform: option,
                }
              )

            case 'dd_mp':
              return new PlatformProxy$1(
                dd,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_dd',
                },
                {
                  mpPlatform: 'dingding',
                  mp: true,
                  platform: option,
                }
              )

            case 'bl_mg':
              return new PlatformProxy$1(
                bl,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_mg',
                },
                {
                  mpPlatform: 'bilibili',
                  platform: option,
                }
              )

            case 'kuaishou_mp':
              return new PlatformProxy$1(
                ks,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_kuaishou',
                },
                {
                  mpPlatform: 'kuaishou',
                  mp: true,
                  platform: option,
                }
              )
            // case 'qtt_mg':
            //     return new PlatformProxy(qttGame.systemInfo, {persistenceName: 'thinkingdata', persistenceNameOld: 'thinkingdata_qtt'}, {mpPlatform: 'qutoutiao', platform: option });
            // case 'linksure_mg':
            //     return new PlatformProxy(wuji, {persistenceName: 'thinkingdata', persistenceNameOld: 'thinkingdata_linksure'}, {mpPlatform: 'linksure', platform: option });

            case 'qh360_mg':
              return new PlatformProxy$1(
                qh,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_qh360',
                },
                {
                  mpPlatform: 'qh360',
                  platform: option,
                }
              )

            case 'tb_mp':
              return new PlatformProxy$1(
                my,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_tb',
                },
                {
                  mpPlatform: 'tb',
                  mp: true,
                  platform: option,
                }
              )

            case 'jd_mp':
              return new PlatformProxy$1(
                jd,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_jd',
                },
                {
                  mpPlatform: 'jd',
                  mp: true,
                  platform: option,
                }
              )

            case 'qh360_mp':
              return new PlatformProxy$1(
                qh,
                {
                  persistenceName: 'thinkingdata',
                  persistenceNameOld: 'thinkingdata_qh360',
                },
                {
                  mpPlatform: 'qh360',
                  mp: true,
                  platform: option,
                }
              )

            case 'WEB':
              return new PlatformProxy.createInstance()
          }
        },
      },
    ]
  )

  return PlatformProxy$1
})()

var PlatformProxyUniapp = /*#__PURE__*/ (function (_PlatformProxy) {
  _inherits(PlatformProxyUniapp, _PlatformProxy)

  var _super = _createSuper(PlatformProxyUniapp)

  function PlatformProxyUniapp() {
    _classCallCheck(this, PlatformProxyUniapp)

    return _super.apply(this, arguments)
  }

  _createClass(
    PlatformProxyUniapp,
    [
      {
        key: 'initAutoTrackInstance',
        value: function initAutoTrackInstance(instance, config) {
          logger$1.info(instance)
          logger$1.info(config)
        },
      },
    ],
    [
      {
        key: 'createInstance',
        value: function createInstance() {
          var proxy = new PlatformProxyUniapp(
            uni,
            {
              persistenceName: 'thinkingdata',
              persistenceNameOld: 'thinkingdata_uniapp',
            },
            {
              mpPlatform: 'uniapp',
              mp: true,
              platform: 'uniapp',
            }
          ) // #ifdef  MP-WEIXIN

          proxy = PlatformProxy$1._createInstance('wechat_mp') // #endif
          // #ifdef  MP-ALIPAY

          proxy = PlatformProxy$1._createInstance('ali_mp') // #endif
          // #ifdef  MP-BAIDU

          proxy = PlatformProxy$1._createInstance('baidu_mp') // #endif
          // #ifdef  MP-TOUTIAO

          proxy = PlatformProxy$1._createInstance('tt_mp') // #endif
          // #ifdef  MP-LARK

          proxy = PlatformProxy$1._createInstance('tt_mp') // #endif
          // #ifdef  MP-QQ

          proxy = PlatformProxy$1._createInstance('qq_mp') // #endif
          // #ifdef  MP-KUAISHOU

          proxy = PlatformProxy$1._createInstance('kuaishou_mp') // #endif
          // #ifdef  MP-JD

          proxy = PlatformProxy$1._createInstance('jd_mp') // #endif
          // #ifdef  MP-360

          proxy = PlatformProxy$1._createInstance('qh360_mp') // #endif
          // #ifdef  H5

          proxy = PlatformProxy$1._createInstance('WEB') // #endif

          return proxy
        },
      },
    ]
  )

  return PlatformProxyUniapp
})(PlatformProxy$1)

var PlatformAPI = /*#__PURE__*/ (function () {
  function PlatformAPI() {
    _classCallCheck(this, PlatformAPI)
  }

  _createClass(PlatformAPI, null, [
    {
      key: '_getCurrentPlatform',
      value: function _getCurrentPlatform() {
        return this.currentPlatform || (this.currentPlatform = PlatformProxyUniapp.createInstance())
      },
      /**
       * Get platform specific configuration: persistenceName required
       */
    },
    {
      key: 'getConfig',
      value: function getConfig() {
        return this._getCurrentPlatform().getConfig()
      },
      /**
       * Get local cache data
       * @param {string} name: cache key
       * @param {boolean} async: enable asynchronous getting cached
       * @param {function} callback: callback when getting data asynchronously, the parameter is an object
       * @return return cached data, it is an object
       */
    },
    {
      key: 'getStorage',
      value: function getStorage(name, async, callback) {
        return this._getCurrentPlatform().getStorage(name, async, callback)
      },
      /**
       * Set local cache data
       * @param {string} name: cache key
       * @param {string} value: JSON string value
       */
    },
    {
      key: 'setStorage',
      value: function setStorage(name, value) {
        return this._getCurrentPlatform().setStorage(name, value)
      },
      /**
       * Delete data in local cache with key
       * @param {*} name: cache key
       */
    },
    {
      key: 'removeStorage',
      value: function removeStorage(name) {
        return this._getCurrentPlatform().removeStorage(name)
      },
      /**
       * Get system information asynchronously
       * @param {object} options: callback when getting completion
       * callback parameter：
       * brand: string, device brand
       * model: string, device model
       * screenWidth: number, screen width, unit px
       * screenHeight: number, screen height, unit px
       * system: string, operating system and version
       * platform: string, client platform
       */
    },
    {
      key: 'getSystemInfo',
      value: function getSystemInfo(options) {
        return this._getCurrentPlatform().getSystemInfo(options)
      },
      /**
       * Get network type asynchronously
       * @param {object} options: callback when getting completion
       * res.networkType string: network type
       */
    },
    {
      key: 'getNetworkType',
      value: function getNetworkType(options) {
        return this._getCurrentPlatform().getNetworkType(options)
      },
      /**
       * Listen for network state change
       * @param {function} callback: callback when network state changing
       */
    },
    {
      key: 'onNetworkStatusChange',
      value: function onNetworkStatusChange(callback) {
        this._getCurrentPlatform().onNetworkStatusChange(callback)
      },
      /**
       * Make a network request
       * @param {object} options: parameters, including:
       *   url       string         server url
       *   data      string/object  request parameters
       *   method    string         HTTP method
       *   success   function       success callback
       *   fail      function       fail callback
       *   complete  function       complete callback
       */
    },
    {
      key: 'request',
      value: function request(options) {
        return this._getCurrentPlatform().request(options)
      },
      /**
       * Initialize the lifecycle monitoring instance
       * @param {ThinkingDataAPI} instance: SDK instance, listen lifecycle of application
       * @param {object} config: auto-tracking events config
       */
    },
    {
      key: 'initAutoTrackInstance',
      value: function initAutoTrackInstance(instance, config) {
        return this._getCurrentPlatform().initAutoTrackInstance(instance, config)
      },
      /**
       * Set instance to global
       * @param {object} instance
       * @param {string} name
       */
    },
    {
      key: 'setGlobal',
      value: function setGlobal(instance, name) {
        if (instance && name) {
          this._getCurrentPlatform().setGlobal(instance, name)
        }
      },
      /**
       * Get system startup information, and register APP cut-off foreground callback
       * @param {function} callback
       */
    },
    {
      key: 'getAppOptions',
      value: function getAppOptions(callback) {
        return this._getCurrentPlatform().getAppOptions(callback)
      },
      /**
       * Toast Debug information
       * @param {string} msg: information to display
       */
    },
    {
      key: 'showDebugToast',
      value: function showDebugToast(msg) {
        this._getCurrentPlatform().showToast(msg)
      },
    },
  ])

  return PlatformAPI
})()

/** @const */

var KEY_NAME_MATCH_REGEX = /^[a-zA-Z][a-zA-Z0-9_]{0,49}$/
var PropertyChecker = /*#__PURE__*/ (function () {
  function PropertyChecker() {
    _classCallCheck(this, PropertyChecker)
  }

  _createClass(PropertyChecker, null, [
    {
      key: 'stripProperties',
      value: function stripProperties(prop) {
        if (!_.isObject(prop)) {
          return prop
        }

        _.each(prop, function (v, k) {
          if (!(_.isString(v) || _.isNumber(v) || _.isDate(v) || _.isBoolean(v) || _.isArray(v) || _.isObject(v))) {
            logger$1.warn('Your data -', k, v, '- format does not meet requirements and may not be stored correctly. Attribute values only support String, Number, Date, Boolean, Array, Object')
          }
        })

        return prop
      },
    },
    {
      key: '_checkPropertiesKey',
      value: function _checkPropertiesKey(obj) {
        var flag = true

        _.each(obj, function (content, key) {
          if (!KEY_NAME_MATCH_REGEX.test(key)) {
            logger$1.warn('Invalid KEY: ' + key)
            flag = false
          }
        })

        return flag
      },
    },
    {
      key: 'event',
      value: function event(s) {
        if (!_.isString(s) || !KEY_NAME_MATCH_REGEX.test(s)) {
          logger$1.warn('Check the parameter format. The eventName must start with an English letter and contain no more than 50 characters including letters, digits, and underscores: ' + s)
          return false
        } else {
          return true
        }
      },
    },
    {
      key: 'propertyName',
      value: function propertyName(s) {
        if (!_.isString(s) || !KEY_NAME_MATCH_REGEX.test(s)) {
          logger$1.warn('Check the parameter format. PropertyName must start with a letter and contain letters, digits, and underscores (_). The value is a string of no more than 50 characters: ' + s)
          return false
        } else {
          return true
        }
      },
    },
    {
      key: 'properties',
      value: function properties(p) {
        this.stripProperties(p)

        if (p) {
          if (_.isObject(p)) {
            if (this._checkPropertiesKey(p)) {
              return true
            } else {
              logger$1.warn('Check the parameter format. The properties key must start with a letter, contain digits, letters, and underscores (_), and contain a maximum of 50 characters')
              return false
            }
          } else {
            logger$1.warn('properties can be none, but it must be an object')
            return false
          }
        } else {
          return true
        }
      },
    },
    {
      key: 'propertiesMust',
      value: function propertiesMust(p) {
        this.stripProperties(p)

        if (p === undefined || !_.isObject(p) || _.isEmptyObject(p)) {
          logger$1.warn('properties must be an object with a value')
          return false
        } else {
          if (this._checkPropertiesKey(p)) {
            return true
          } else {
            logger$1.warn('Check the parameter format. The properties key must start with a letter, contain digits, letters, and underscores (_), and contain a maximum of 50 characters')
            return false
          }
        }
      },
    },
    {
      key: 'userId',
      value: function userId(id) {
        if (_.isString(id) && /^.{1,64}$/.test(id)) {
          return true
        } else {
          logger$1.warn('The user ID must be a string of less than 64 characters and cannot be null')
          return false
        }
      },
    },
    {
      key: 'userAddProperties',
      value: function userAddProperties(p) {
        if (!this.propertiesMust(p)) return false

        for (var i in p) {
          if (!_.isNumber(p[i])) {
            logger$1.warn('The attributes of userAdd need to be Number')
            return false
          }
        }

        return true
      },
    },
    {
      key: 'userAppendProperties',
      value: function userAppendProperties(p) {
        if (!this.propertiesMust(p)) return false

        for (var i in p) {
          if (!_.isArray(p[i])) {
            logger$1.warn('The attribute of userAppend must be Array')
            return false
          }
        }

        return true
      },
    },
  ])

  return PropertyChecker
})()

var HttpTask = /*#__PURE__*/ (function () {
  function HttpTask(data, serverUrl, tryCount, timeout, callback) {
    _classCallCheck(this, HttpTask)

    this.data = data
    this.serverUrl = serverUrl
    this.callback = callback
    this.tryCount = _.isNumber(tryCount) ? tryCount : 1
    this.timeout = _.isNumber(timeout) ? timeout : 3000
    this.taClassName = 'HttpTask'
  }

  _createClass(HttpTask, [
    {
      key: 'run',
      value: function run() {
        var _this = this

        var headers = _.createExtraHeaders()

        headers['content-type'] = 'application/json' // eslint-disable-next-line no-undef

        var request = PlatformAPI.request({
          url: this.serverUrl,
          method: 'POST',
          data: this.data,
          header: headers,
          success: function success(res) {
            _this.onSuccess(res)

            clearTimeout(timer)
          },
          fail: function fail(res) {
            _this.onFailed(res)

            clearTimeout(timer)
          },
        })
        var timer = setTimeout(function () {
          if ((_.isObject(request) || _.isPromise(request)) && _.isFunction(request.abort)) {
            request.abort()
          }
        }, this.timeout)
      },
    },
    {
      key: 'onSuccess',
      value: function onSuccess(res) {
        if (res.statusCode === 200) {
          var msg

          switch (res.data.code) {
            case 0:
              msg = 'success'
              break

            case -1:
              msg = 'invalid data'
              break

            case -2:
              msg = 'invalid APP ID'
              break

            default:
              msg = 'Unknown return code'
          }

          this.callback({
            code: res.data.code,
            msg: msg,
          })
        } else {
          this.callback({
            code: -3,
            msg: res.statusCode,
          })
        }
      },
    },
    {
      key: 'onFailed',
      value: function onFailed(res) {
        if (--this.tryCount > 0) {
          this.run()
        } else {
          this.callback({
            code: -3,
            msg: res.errMsg,
          })
        }
      },
    },
  ])

  return HttpTask
})()

var HttpTaskDebug = /*#__PURE__*/ (function () {
  function HttpTaskDebug(data, serverDebugUrl, tryCount, timeout, dryrun, deviceId, callback) {
    _classCallCheck(this, HttpTaskDebug)

    this.data = data
    this.serverDebugUrl = serverDebugUrl
    this.callback = callback
    this.tryCount = _.isNumber(tryCount) ? tryCount : 1
    this.timeout = _.isNumber(timeout) ? timeout : 3000
    this.dryrun = dryrun
    this.deviceId = deviceId
    this.taClassName = 'HttpTaskDebug'
  }

  _createClass(HttpTaskDebug, [
    {
      key: 'run',
      value: function run() {
        var _this2 = this

        var debugData = 'appid=' + this.data['#app_id'] + '&source=client&dryRun=' + this.dryrun + '&deviceId=' + this.deviceId + '&data=' + encodeURIComponent(JSON.stringify(this.data['data'][0]))

        var headers = _.createExtraHeaders()

        headers['content-type'] = 'application/x-www-form-urlencoded' // eslint-disable-next-line no-undef

        var request = PlatformAPI.request({
          url: this.serverDebugUrl,
          method: 'POST',
          data: debugData,
          header: headers,
          success: function success(res) {
            _this2.onSuccess(res)

            clearTimeout(timer)
          },
          fail: function fail(res) {
            _this2.onFailed(res)

            clearTimeout(timer)
          },
        })
        var timer = setTimeout(function () {
          if ((_.isObject(request) || _.isPromise(request)) && _.isFunction(request.abort)) {
            request.abort()
          }
        }, this.timeout)
      },
    },
    {
      key: 'onSuccess',
      value: function onSuccess(res) {
        if (res.statusCode === 200) {
          var msg

          if (res.data['errorLevel'] === 0) {
            msg = 'Verify data success.'
          } else if (res.data['errorLevel'] === 1) {
            var errorProperties = res.data['errorProperties']
            var errorStr = ''

            for (var i = 0; i < errorProperties.length; i++) {
              var errorReasons = errorProperties[i]['errorReason']
              var propertyName = errorProperties[i]['propertyName']
              errorStr = errorStr + ' propertyName:' + propertyName + ' errorReasons:' + errorReasons + '\n'
            }

            msg = 'Debug data error. errorLevel:' + res.data['errorLevel'] + ' reason:' + errorStr
          } else if (res.data['errorLevel'] === 2 || res.data['errorLevel'] === -1) {
            msg = 'Debug data error. errorLevel:' + res.data['errorLevel'] + ' reason:' + res.data['errorReasons']
          }

          logger$1.info(msg)
          this.callback({
            code: res.data['errorLevel'],
            msg: msg,
          })
        } else {
          this.callback({
            code: -3,
            msg: res.statusCode,
          })
        }
      },
    },
    {
      key: 'onFailed',
      value: function onFailed(res) {
        if (--this.tryCount > 0) {
          this.run()
        } else {
          this.callback({
            code: -3,
            msg: res.errMsg,
          })
        }
      },
    },
  ])

  return HttpTaskDebug
})()

var SenderQueue = /*#__PURE__*/ (function () {
  function SenderQueue() {
    _classCallCheck(this, SenderQueue)

    this.items = []
    this.isRunning = false
    this.showDebug = false
  }

  _createClass(SenderQueue, [
    {
      key: 'enqueue',
      value: function enqueue(data, serverUrl, config) {
        var _enqueue = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true

        var element
        var that = this

        if (config.debugMode === 'debug') {
          element = new HttpTaskDebug(data, serverUrl, config.maxRetries, config.sendTimeout, 0, config.deviceId, function (res) {
            that.isRunning = false

            if (_.isFunction(config.callback)) {
              config.callback(res)
            }

            that._runNext()

            if (that.showDebug === false) {
              if (res.code === 0 || res.code === 1 || res.code === 2) {
                that.showDebug = true // eslint-disable-next-line no-undef

                if (_.isFunction(PlatformAPI.showDebugToast)) {
                  // eslint-disable-next-line no-undef
                  PlatformAPI.showDebugToast('The current mode is Debug')
                }
              }
            }
          })
        } else if (config.debugMode === 'debugOnly') {
          element = new HttpTaskDebug(data, serverUrl, config.maxRetries, config.sendTimeout, 1, config.deviceId, function (res) {
            that.isRunning = false

            if (_.isFunction(config.callback)) {
              config.callback(res)
            }

            that._runNext()

            if (that.showDebug === false) {
              if (res.code === 0 || res.code === 1 || res.code === 2) {
                that.showDebug = true // eslint-disable-next-line no-undef

                if (_.isFunction(PlatformAPI.showDebugToast)) {
                  // eslint-disable-next-line no-undef
                  PlatformAPI.showDebugToast('The current mode is debugOnly')
                }
              }
            }
          })
        } else {
          element = new HttpTask(JSON.stringify(data), serverUrl, config.maxRetries, config.sendTimeout, function (res) {
            that.isRunning = false

            if (_.isFunction(config.callback)) {
              config.callback(res)
            }

            that._runNext()
          })
        }

        if (_enqueue === true) {
          this.items.push(element)

          this._runNext()
        } else {
          element.run()
        }
      },
    },
    {
      key: '_dequeue',
      value: function _dequeue() {
        return this.items.shift()
      },
    },
    {
      key: '_runNext',
      value: function _runNext() {
        if (this.items.length > 0 && !this.isRunning) {
          this.isRunning = true

          if (this.items[0].taClassName !== 'HttpTask') {
            this._dequeue().run()
          } else {
            var items = this.items.splice(0, this.items.length)
            var httpTask0 = items[0]
            var data = JSON.parse(httpTask0.data)
            var appId = data['#app_id']
            var callbackList = []
            callbackList.push(httpTask0.callback)

            for (var i = 1; i < items.length; i++) {
              var task = items[i]
              var taskData = JSON.parse(task.data)

              if (taskData['#app_id'] === appId && httpTask0.serverUrl === task.serverUrl) {
                data['data'] = data['data'].concat(taskData['data'])
                callbackList.push(task.callback)
              } else {
                // If serverUrl and appId is different, it needs to be put back into the queue and sent next time
                this.items.push(task)
              }
            }

            var flushTime = new Date().getTime()
            data['#flush_time'] = flushTime
            var element
            element = new HttpTask(JSON.stringify(data), httpTask0.serverUrl, httpTask0.tryCount, httpTask0.timeout, function (res) {
              for (var cb in callbackList) {
                if (Object.hasOwnProperty.call(callbackList, cb)) {
                  var _element = callbackList[cb]

                  _element(res)
                }
              }
            })
            element.run()
          }
        }
      },
    },
  ])

  return SenderQueue
})()

var senderQueue = new SenderQueue()

var DEFAULT_CONFIG = {
  name: 'thinkingdata',
  // global name
  // eslint-disable-next-line camelcase
  is_plugin: false,
  // if is it plugin. Basic library < 2.6.4 does not allow modification of App and Page
  maxRetries: 3,
  // number of retries for data reporting requests. v1.3.0+
  sendTimeout: 3000,
  // request timeout, Ms
  enablePersistence: true,
  // enable local storage
  asyncPersistence: false,
  // enable asynchronous storage
  enableLog: true,
  // enable printing logs
  strict: false,
  // disable strict data format checking, allow possible problem data to be reported
  debugMode: 'none',
  // Debug mode (none/debug/debugOnly)
  enableCalibrationTime: false,
}
/**
 * Get system information asynchronously and initialize preset properties
 *
 * #lib: SDK type,
 * #lib_version: SDK version
 * #network_type: current network type
 * #manufacture: device manufactory
 * #device_model: device mode, e.g iPhone 8
 * #screen_width: device screen width
 * #screen_height: device screen height
 * #os: device os name
 * #os_version: device os version
 * #mp_platform: current platform name
 */

var systemInformation = {
  properties: {
    '#lib': Config.LIB_NAME,
    '#lib_version': Config.LIB_VERSION,
  },
  initDeviceId: function initDeviceId(deviceId) {
    if (_.isString(deviceId)) {
      this.properties['#device_id'] = deviceId
    }
  },
  getSystemInfo: function getSystemInfo(callback) {
    var that = this
    PlatformAPI.onNetworkStatusChange(function (res) {
      that.properties['#network_type'] = res.networkType
    })
    PlatformAPI.getNetworkType({
      success: function success(res) {
        that.properties['#network_type'] = res.networkType
      },
      complete: function complete() {
        PlatformAPI.getSystemInfo({
          success: function success(res) {
            var osInfo = res['system'] ? res['system'].replace(/\s+/g, ' ').split(' ') : []
            var data = {
              '#manufacturer': res['brand'],
              '#device_model': res['model'],
              '#screen_width': Number(res['screenWidth']),
              '#screen_height': Number(res['screenHeight']),
              '#os': osInfo[0],
              '#os_version': osInfo[1],
              '#mp_platform': res['mp_platform'],
              '#system_language': res['systemLanguage'], //'#utm': _.getUtm()
            }

            _.extend(that.properties, data)

            _.setMpPlatform(res['mp_platform'])
          },
          complete: function complete() {
            callback()
          },
        })
      },
    })
  },
}
/**
 * Data cache management class
 *
 * Keys :
 * 1. device_id: #device_id
 * 2. distinct_id: #distinct_id
 * 3. account_id: #account_id
 * 4. props: super properties
 * 5. event_timers: #duration
 *
 */

var ThinkingDataPersistence = /*#__PURE__*/ (function () {
  function ThinkingDataPersistence(config, callback) {
    var _this = this

    _classCallCheck(this, ThinkingDataPersistence)

    this.enabled = config.enablePersistence

    if (this.enabled) {
      if (config.isChildInstance) {
        this.name = config.persistenceName + '_' + config.name
        this.nameOld = config.persistenceNameOld + '_' + config.name
      } else {
        this.name = config.persistenceName
        this.nameOld = config.persistenceNameOld
      }

      if (config.asyncPersistence) {
        this._state = {}
        PlatformAPI.getStorage(this.name, true, function (data) {
          if (_.isEmptyObject(data)) {
            PlatformAPI.getStorage(_this.nameOld, true, function (dataOld) {
              _this._state = _.extend2Layers({}, dataOld, _this._state)

              _this._init(config, callback)

              _this._save()
            })
          } else {
            _this._state = _.extend2Layers({}, data, _this._state)

            _this._init(config, callback)

            _this._save()
          }
        })
      } else {
        this._state = PlatformAPI.getStorage(this.name) || {}

        if (_.isEmptyObject(this._state)) {
          this._state = PlatformAPI.getStorage(this.nameOld) || {}
        }

        this._init(config, callback)
      }
    } else {
      this._state = {}

      this._init(config, callback)
    }
  }

  _createClass(ThinkingDataPersistence, [
    {
      key: '_init',
      value: function _init(config, callback) {
        if (!this.getDistinctId()) {
          this.setDistinctId(_.UUID())
        }

        if (!config.isChildInstance) {
          if (!this.getDeviceId()) {
            this._setDeviceId(_.UUID())
          }

          systemInformation.initDeviceId(this.getDeviceId())
        } // Mark sdk initialization is complete, you can write to the local cache

        this.initComplete = true

        if (typeof callback === 'function') {
          callback()
        }

        this._save()
      },
    },
    {
      key: '_save',
      value: function _save() {
        if (this.enabled && this.initComplete) {
          PlatformAPI.setStorage(this.name, JSON.stringify(this._state))
        }
      },
    },
    {
      key: '_set',
      value: function _set(name, value) {
        var _this2 = this

        var obj

        if (typeof name === 'string') {
          obj = {}
          obj[name] = value
        } else if (_typeof(name) === 'object') {
          obj = name
        }

        _.each(obj, function (value, key) {
          _this2._state[key] = value
        })

        this._save()
      },
    },
    {
      key: '_get',
      value: function _get(name) {
        return this._state[name]
      },
    },
    {
      key: 'setEventTimer',
      value: function setEventTimer(eventName, timestamp) {
        var timers = this._state.event_timers || {}
        timers[eventName] = timestamp

        this._set('event_timers', timers)
      },
    },
    {
      key: 'removeEventTimer',
      value: function removeEventTimer(eventName) {
        var timers = this._state.event_timers || {}
        var timestamp = timers[eventName]

        if (!_.isUndefined(timestamp)) {
          delete this._state.event_timers[eventName]

          this._save()
        }

        return timestamp
      },
    },
    {
      key: 'getDeviceId',
      value: function getDeviceId() {
        return this._state.device_id
      },
    },
    {
      key: '_setDeviceId',
      value: function _setDeviceId(deviceId) {
        if (this.getDeviceId()) {
          logger$1.warn('cannot modify the device id.')
          return
        }

        this._set('device_id', deviceId)
      },
    },
    {
      key: 'getDistinctId',
      value: function getDistinctId() {
        return this._state.distinct_id
      },
    },
    {
      key: 'setDistinctId',
      value: function setDistinctId(distinctId) {
        this._set('distinct_id', distinctId)
      },
    },
    {
      key: 'getAccountId',
      value: function getAccountId() {
        return this._state.account_id
      },
    },
    {
      key: 'setAccountId',
      value: function setAccountId(accoundId) {
        this._set('account_id', accoundId)
      },
    },
    {
      key: 'getSuperProperties',
      value: function getSuperProperties() {
        return this._state.props || {}
      },
    },
    {
      key: 'setSuperProperties',
      value: function setSuperProperties(superProperties, replace) {
        var props = replace ? superProperties : _.extend(this.getSuperProperties(), superProperties)

        this._set('props', props)
      },
    },
  ])

  return ThinkingDataPersistence
})()

var dataStoragePrefix = 'tampsdk_'
var tabStoragePrefix = 'tab_tampsdk_'

var BatchConsumer = /*#__PURE__*/ (function () {
  function BatchConsumer(config, ta) {
    _classCallCheck(this, BatchConsumer)

    this.config = config
    this.ta = ta
    this.timer = null
    this.batchConfig = _.extend(
      {
        size: 5,
        //event batch size
        interval: 5000,
        //interval to send data in milliseconds
        storageLimit: 200, // event cache maximum limit
      },
      this.config.batchConfig
    )

    if (this.batchConfig.size < 1) {
      this.batchConfig.size = 1
    }

    if (this.batchConfig.size > 30) {
      this.batchConfig.size = 30
    }

    this.tabKey = tabStoragePrefix + this.config.appId
    this.storageLimit = this.batchConfig['storageLimit']
    this.isRequest = false
    this.trackList = []
    this.needFlush = false
  }

  _createClass(BatchConsumer, [
    {
      key: 'batchInterval',
      value: function batchInterval() {
        var self = this
        self.timer = setTimeout(function () {
          self.recycle()
          self.send()
          clearTimeout(self.timer)
          self.batchInterval()
        }, this.batchConfig.interval)
      },
    },
    {
      key: 'add',
      value: function add(data) {
        if (this.isRequest) {
          this.trackList.push(data)
          return
        }

        var d = data //if enable batch send, modify time_calibration to 5
        // if (d['properties']['#time_calibration'] === 3) {
        //     d['properties']['#time_calibration'] = 5;
        // }

        var dataKey = dataStoragePrefix + this.config.appId + '_' + String(_.UUID())
        var tabStorage = PlatformAPI.getStorage(this.tabKey)

        if (!_.isArray(tabStorage)) {
          tabStorage = []
        } // if (tabStorage === null) {
        //     tabStorage = [];
        // }

        if (tabStorage.length <= this.storageLimit) {
          //If the data in the cache does not reach the maximum number, save
          tabStorage.push(dataKey)
          PlatformAPI.setStorage(this.tabKey, JSON.stringify(tabStorage))
          PlatformAPI.setStorage(dataKey, JSON.stringify(d))
        } else {
          //Delete 20 items first, then save
          var deleteDatas = tabStorage.splice(0, 20)
          console.log('deleted events data:' + deleteDatas)
          tabStorage.push(dataKey)
          PlatformAPI.setStorage(this.tabKey, JSON.stringify(tabStorage))
          PlatformAPI.setStorage(dataKey, JSON.stringify(d))
          var postData = {}
          var dList = []

          for (var i = 0; i < deleteDatas.length; i++) {
            var item = PlatformAPI.getStorage(deleteDatas[i])
            dList.push(item)
          }

          postData['data'] = dList
          postData['#app_id'] = this.config['appId']
          this.request(postData, deleteDatas)
        }
      },
    },
    {
      key: 'flush',
      value: function flush() {
        clearTimeout(this.timer)
        this.send()
        this.batchInterval()
      },
    },
    {
      key: 'send',
      value: function send() {
        if (this.isRequest) {
          this.needFlush = true
          return
        }

        var tabStorage = PlatformAPI.getStorage(this.tabKey)

        if (tabStorage) {
          if (tabStorage.length) {
            var postData = {}
            var data = []
            var tabList = []
            var len = tabStorage.length < this.batchConfig.size ? tabStorage.length : this.batchConfig.size

            for (var i = 0; i < len; i++) {
              var d = PlatformAPI.getStorage(tabStorage[i])
              data.push(d)
              tabList.push(tabStorage[i])
            }

            postData['data'] = data
            postData['#app_id'] = this.config['appId']
            this.request(postData, tabList)
          }
        }
      },
    },
    {
      key: 'request',
      value: function request(data, dataKeys) {
        var self = this
        logger$1.info('Flush data: ' + JSON.stringify(data))
        self.isRequest = true
        senderQueue.enqueue(data, this.ta.serverUrl, {
          maxRetries: this.config.maxRetries,
          sendTimeout: this.config.sendTimeout,
          callback: function callback(res) {
            if (res.code === 0) {
              self.remove(dataKeys)
            }

            self.isRequest = false

            for (var i = 0; i < self.trackList.length; i++) {
              self.add(self.trackList[i])
            }

            self.trackList = []

            if (self.needFlush) {
              self.needFlush = false
              self.flush()
            }
          },
          debugMode: this.config.debugMode,
          deviceId: this.ta.getDeviceId(),
        })
      },
    },
    {
      key: 'remove',
      value: function remove(dataKeys) {
        var tabStorage = PlatformAPI.getStorage(this.tabKey)

        if (tabStorage) {
          for (var i = 0; i < dataKeys.length; i++) {
            var idx = _.indexOf(tabStorage, dataKeys[i])

            if (idx > -1) {
              tabStorage.splice(idx, 1)
            }

            PlatformAPI.removeStorage(dataKeys[i])
          }

          PlatformAPI.setStorage(this.tabKey, JSON.stringify(tabStorage))
        }
      },
    },
    {
      key: 'recycle',
      value: function recycle() {
        //localStorage is for web
        // var tabStorage = PlatformAPI.getStorage(this.tabKey);
        // if (tabStorage) {
        //     if (tabStorage.length === 0) {
        //         for (var i = 0; i < localStorage.length; i++) {
        //             var key = localStorage.key(i);
        //             if (key.indexOf(dataStoragePrefix + this.config['appId']) === 0) {
        //                 tabStorage.push(key);
        //             }
        //         }
        //         if (tabStorage.length > 0) {
        //             PlatformAPI.setStorage(this.tabKey, JSON.stringify(tabStorage));
        //         }
        //     }
        // }
      },
    },
  ])

  return BatchConsumer
})()

var ThinkingDataAPI = /*#__PURE__*/ (function () {
  function ThinkingDataAPI(config) {
    _classCallCheck(this, ThinkingDataAPI)

    config.appId = config.appId ? _.checkAppId(config.appId) : _.checkAppId(config.appid)
    config.serverUrl = config.serverUrl ? _.checkUrl(config.serverUrl) : _.checkUrl(config.server_url)

    var defaultConfig = _.extend({}, DEFAULT_CONFIG, PlatformAPI.getConfig())

    if (_.isObject(config)) {
      this.config = _.extend(defaultConfig, config)
    } else {
      this.config = defaultConfig
    }

    this._init(this.config)
  } // internal init function. it should not be used by users.

  _createClass(ThinkingDataAPI, [
    {
      key: '_init',
      value: function _init(config) {
        var _this3 = this

        this.name = config.name
        this.appId = config.appId || config.appid
        var serverUrl = config.serverUrl || config.server_url
        this.serverUrl = serverUrl + '/sync_xcx'
        this.serverDebugUrl = serverUrl + '/data_debug'
        this.configUrl = serverUrl + '/config'
        this.autoTrackProperties = {} // cache commands.

        this._queue = []
        this.updateConfig(this.configUrl, this.appId)

        if (config.isChildInstance) {
          this._state = {}
        } else {
          logger$1.enabled = config.enableLog
          this.instances = []
          this._state = {
            getSystemInfo: false,
            initComplete: false,
          } // systemInformation.getSystemInfo(() => {
          //     this._updateState({
          //         getSystemInfo: true,
          //     });
          // });

          PlatformAPI.setGlobal(this, this.name)
        }

        this.store = new ThinkingDataPersistence(config, function () {
          if (_this3.config.asyncPersistence && _.isFunction(_this3.config.persistenceComplete)) {
            _this3.config.persistenceComplete(_this3)
          }

          _this3._updateState()
        })
        this.enabled = _.isBoolean(this.store._get('ta_enabled')) ? this.store._get('ta_enabled') : true
        this.isOptOut = _.isBoolean(this.store._get('ta_isOptOut')) ? this.store._get('ta_isOptOut') : false

        if (!config.isChildInstance && config.autoTrack) {
          this.autoTrack = PlatformAPI.initAutoTrackInstance(this, config)
        } //Enable batch reporting of data

        if (this.config.enableBatch !== undefined && this.config.enableBatch !== false) {
          this.batchConsumer = new BatchConsumer(this.config, this)
          this.batchConsumer.batchInterval()
        }
      },
    },
    {
      key: 'initSystemInfo',
      value: function initSystemInfo() {
        var _this4 = this

        if (!this.config.isChildInstance) {
          systemInformation.getSystemInfo(function () {
            _this4._updateState({
              getSystemInfo: true,
            })
          })
        }
      },
    },
    {
      key: 'updateConfig',
      value: function updateConfig(configUrl, appId) {
        var _this5 = this

        var headers = _.createExtraHeaders()

        headers['content-type'] = 'application/json'
        var request = PlatformAPI.request({
          url: configUrl + '?appid=' + appId,
          method: 'GET',
          header: headers,
          success: function success(res) {
            if (!_.isUndefined(res) && !_.isUndefined(res.data)) {
              logger$1.info('Get remote config success' + '(' + appId + ') :' + JSON.stringify(res.data))

              if (!_.isUndefined(res.data['data'])) {
                _this5.config.syncBatchSize = res.data['data']['sync_batch_size']
                _this5.config.syncInterval = res.data['data']['sync_interval']
                _this5.config.disableEventList = res.data['data']['disable_event_list']

                if (!_.isUndefined(res.data['data']['secret_key'])) {
                  var secretKey = res.data['data']['secret_key']
                  _this5.config.secretKey = {
                    publicKey: secretKey['key'],
                    version: secretKey['version'],
                  }
                }
              }
            }
          },
          fail: function fail(res) {
            logger$1.info('Get remote config fail' + '(' + appId + ') :' + res.errMsg)
          },
        })
        setTimeout(function () {
          if ((_.isObject(request) || _.isPromise(request)) && _.isFunction(request.abort)) {
            request.abort()
          }
        }, 3000)
      },
      /**
       * Create a new instance (sub-instance).
       * All properties that can be set are independent of the main instance.
       *
       * @param {string} name: sub-instance name
       * @param {object} config: optional, config of sub-instance
       */
    },
    {
      key: 'initInstance',
      value: function initInstance(name, config) {
        if (this.config.isChildInstance) {
          logger$1.warn('initInstance() cannot be called on child instance')
          return undefined
        }

        if (_.isString(name) && name !== this.name && _.isUndefined(this[name])) {
          var instance = new ThinkingDataAPI(
            _.extend(
              {},
              this.config,
              {
                enablePersistence: false,
                isChildInstance: true,
                name: name,
              },
              config
            )
          )
          this[name] = instance
          this.instances.push(name)
          this[name]._state = this._state
          return instance
        } else {
          logger$1.warn('initInstance() failed due to the name is invalid: ' + name)
          return undefined
        }
      },
      /**
       * Get sub-instance with name
       *
       * @param {string} name: sub-instance name
       */
    },
    {
      key: 'lightInstance',
      value: function lightInstance(name) {
        return this[name]
      },
      /**
       * Internal function, used to set some preset properties related to the life cycle
       *
       * Common attributes are:
       * - #scene Scene value, available in onLaunch callback. It can also be actively obtained according to the platform interface
       * - #url_path page path
       *
       * When sending event data, the priority is:
       * Event Properties > Dynamic Public Properties > Common Event Properties > Automatic Collection Properties > Other Preset Properties
       *
       * @param {object} props, event properties
       */
    },
    {
      key: '_setAutoTrackProperties',
      value: function _setAutoTrackProperties(props) {
        _.extend(this.autoTrackProperties, props)
      },
      /**
       * After calling init(), the data starts to be reported
       *
       * Before calling this function, all reporting requests will be cached. When the user completes the necessary settings, call this function to trigger reporting.
       */
    },
    {
      key: 'init',
      value: function init() {
        this.initSystemInfo()
        if (this._state.initComplete) return false

        this._updateState({
          initComplete: true,
        })

        logger$1.info('TDAnalytics SDK initialize success, AppId = ' + this.config.appId + ', ServerUrl = ' + this.config.serverUrl + ', Mode = ' + this.config.model + ', DeviceId = ' + this.getDeviceId() + ', Lib = ' + Config.LIB_NAME + ', LibVersion = ' + Config.LIB_VERSION)
      },
      /**
       * Internal function, to judge whether the initialization is completed, and the data can be actually sent
       *
       * Each instance has three asynchronous states
       * - getSystemInfo Whether has obtained system information, the sub-instance defaults to true.
       * - initComplete Whether the init() function is called, true means that the user has completed the necessary initialization settings. Child instances this state defaults to true.
       * - store.initComplete Whether the cache information has been read
       */
    },
    {
      key: '_isReady',
      value: function _isReady() {
        return this._state.getSystemInfo && this._state.initComplete && this.store.initComplete && this.getDeviceId()
      },
    },
    {
      key: '_updateState',
      value: function _updateState(state) {
        var _this6 = this

        if (_.isObject(state)) {
          _.extend(this._state, state)
        }

        this._onStateChange()

        _.each(this.instances, function (name) {
          _this6[name]._onStateChange()
        })
      }, // Only after the system information initialization is completed and the user actively calls init(), will the data be actually sent.
    },
    {
      key: '_onStateChange',
      value: function _onStateChange() {
        var _this7 = this

        if (this._isReady() && this._queue && this._queue.length > 0) {
          _.each(this._queue, function (item) {
            _this7[item[0]].apply(_this7, slice.call(item[1]))
          })

          this._queue = []
        }
      },
    },
    {
      key: '_hasDisabled',
      value: function _hasDisabled() {
        var hasDisabled = !this.enabled || this.isOptOut

        if (hasDisabled) {
          logger$1.info('SDK is Pause or Stop!')
        }

        return hasDisabled
      }, // send request. Due to the limitations of some platforms on the number of network connections, we use senderQueue to send data.
    },
    {
      key: '_sendRequest',
      value: function _sendRequest(eventData, time, tryBeacon) {
        if (this._hasDisabled()) {
          return
        }

        if (!_.isUndefined(this.config.disableEventList)) {
          if (this.config.disableEventList.includes(eventData.eventName)) {
            logger$1.info('Disabled Event : ' + eventData.eventName)
            return
          }
        }

        time = _.isDate(time) ? time : new Date()
        var data = {
          data: [
            {
              '#type': eventData.type,
              '#time': _.formatDate(time),
              '#distinct_id': this.store.getDistinctId(),
            },
          ],
        }

        if (this.store.getAccountId()) {
          data.data[0]['#account_id'] = this.store.getAccountId()
        }

        if (eventData.type === 'track' || eventData.type === 'track_update' || eventData.type === 'track_overwrite') {
          data.data[0]['#event_name'] = eventData.eventName

          if (eventData.type === 'track_update' || eventData.type === 'track_overwrite') {
            data.data[0]['#event_id'] = eventData.extraId
          } else if (eventData.firstCheckId) {
            data.data[0]['#first_check_id'] = eventData.firstCheckId
          }

          data.data[0]['properties'] = _.extend(
            {
              '#zone_offset': 0 - time.getTimezoneOffset() / 60.0,
            },
            systemInformation.properties,
            this.autoTrackProperties,
            this.store.getSuperProperties(),
            this.dynamicProperties ? this.dynamicProperties() : {}
          )
          var startTimestamp = this.store.removeEventTimer(eventData.eventName)

          if (!_.isUndefined(startTimestamp)) {
            var durationMillisecond = new Date().getTime() - startTimestamp
            var duration = parseFloat((durationMillisecond / 1000).toFixed(3))

            if (duration > 86400) {
              duration = 86400
            } else if (duration < 0) {
              duration = 0
            }

            data.data[0]['properties']['#duration'] = duration
          }
        } else {
          data.data[0]['properties'] = {}
        }

        if (_.isObject(eventData.properties) && !_.isEmptyObject(eventData.properties)) {
          _.extend(data.data[0].properties, eventData.properties)
        }

        _.searchObjDate(data.data[0])

        if (this.config.maxRetries > 1) {
          data.data[0]['#uuid'] = _.UUIDv4()
        }

        data['#app_id'] = this.appId
        logger$1.info('Tracking data, ' + JSON.stringify(data, null, 4))
        var serverUrl = this.config.debugMode === 'debug' || this.config.debugMode === 'debugOnly' ? this.serverDebugUrl : this.serverUrl

        if (_.isBoolean(this.config.enableEncrypt) && this.config.enableEncrypt === true) {
          data.data[0] = _.generateEncryptyData(data.data[0], this.config.secretKey)
        }

        if (this.batchConsumer && this.config.debugMode === 'none' && !tryBeacon) {
          this.batchConsumer.add(data.data[0])

          if (_.isFunction(eventData.onComplete)) {
            eventData.onComplete({
              code: 0,
              msg: 'success',
            })
          }

          return
        }

        if (tryBeacon) {
          var formData = new FormData()

          if (this.config.debugMode === 'debug' || this.config.debugMode === 'debugOnly') {
            formData.append('source', 'client')
            formData.append('appid', this.appId)
            formData.append('dryRun', this.config.debugMode === 'debugOnly' ? 1 : 0)
            formData.append('deviceId', this.getDeviceId())
            formData.append('data', JSON.stringify(data.data[0]))
            navigator.sendBeacon(serverUrl, formData)
          } else {
            var flushTime = new Date().getTime()
            data['#flush_time'] = flushTime
            navigator.sendBeacon(serverUrl, JSON.stringify(data))
          }

          if (_.isFunction(eventData.onComplete)) {
            eventData.onComplete({
              statusCode: 200,
            })
          }
        } else {
          senderQueue.enqueue(data, serverUrl, {
            maxRetries: this.config.maxRetries,
            sendTimeout: this.config.sendTimeout,
            callback: eventData.onComplete,
            debugMode: this.config.debugMode,
            deviceId: this.getDeviceId(),
          })
        }
      }, // Is it a parameter object
    },
    {
      key: '_isObjectParams',
      value: function _isObjectParams(obj) {
        return _.isObject(obj) && _.isFunction(obj.onComplete)
      },
      /**
       * Track a narmal Event.
       * @param {string} eventName: event name, required
       * @param {object} properties: event properties, optional
       * @param {date} time: event time, optional
       * @param {function} onComplete: callback, optional
       */
    },
    {
      key: 'track',
      value: function track(eventName, properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(eventName)) {
          var options = eventName
          eventName = options.eventName
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if ((PropertyChecker.event(eventName) && PropertyChecker.properties(properties)) || !this.config.strict) {
          this._internalTrack(eventName, properties, time, onComplete)
        } else if (_.isFunction(onComplete)) {
          onComplete({
            code: -1,
            msg: 'invalid parameters',
          })
        }
      },
      /**
       * Track a updatable Event
       * @param {object} options: event infomations
       *
       * options.eventName: event name, required
       * options.eventId: event ID, to mark the event, required
       * options.time: event time, optional
       * options.properties: event properties, optional
       * options.onComplete: callback, optional
       */
    },
    {
      key: 'trackUpdate',
      value: function trackUpdate(options) {
        if (this._hasDisabled()) {
          return
        }

        if (options && options.eventId && ((PropertyChecker.event(options.eventName) && PropertyChecker.properties(options.properties)) || !this.config.strict)) {
          if (this._isReady()) {
            var property = _.checkCalibration(options.properties, options.time, this.config.enableCalibrationTime)

            var time = _.isDate(options.time) ? options.time : new Date()

            this._sendRequest(
              {
                type: 'track_update',
                eventName: options.eventName,
                properties: property,
                onComplete: options.onComplete,
                extraId: options.eventId,
              },
              time
            )
          } else {
            //options.time = time;
            this._queue.push(['trackUpdate', [options]])
          }
        } else {
          logger$1.warn('Invalide parameter for trackUpdate: you should pass an object contains eventId to trackUpdate()')

          if (_.isFunction(options.onComplete)) {
            options.onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Track a overwritable Event
       * @param {object} options event infomations
       *
       * options.eventName: event name, required
       * options.eventId: event ID, to mark the event, required
       * options.time: event time, optional
       * options.properties: event properties, optional
       * options.onComplete: callback, optional
       */
    },
    {
      key: 'trackOverwrite',
      value: function trackOverwrite(options) {
        if (this._hasDisabled()) {
          return
        }

        if (options && options.eventId && ((PropertyChecker.event(options.eventName) && PropertyChecker.properties(options.properties)) || !this.config.strict)) {
          if (this._isReady()) {
            var property = _.checkCalibration(options.properties, options.time, this.config.enableCalibrationTime)

            var time = _.isDate(options.time) ? options.time : new Date()

            this._sendRequest(
              {
                type: 'track_overwrite',
                eventName: options.eventName,
                properties: property,
                onComplete: options.onComplete,
                extraId: options.eventId,
              },
              time
            )
          } else {
            //options.time = time;
            this._queue.push(['trackOverwrite', [options]])
          }
        } else {
          logger$1.warn('Invalide parameter for trackOverwrite: you should pass an object contains eventId to trackOverwrite()')

          if (_.isFunction(options.onComplete)) {
            options.onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Track a first Event
       * @param {object} options event infomations
       *
       * options.eventName: event name, required
       * options.firstCheckId: event ID, to mark the event, default is #device_id, required
       * options.time: event time, optional
       * options.properties: event properties, optional
       * options.onComplete: callback, optional
       */
    },
    {
      key: 'trackFirstEvent',
      value: function trackFirstEvent(options) {
        if (this._hasDisabled()) {
          return
        }

        if (options && options.eventName && ((PropertyChecker.event(options.eventName) && PropertyChecker.properties(options.properties)) || !this.config.strict)) {
          if (this._isReady()) {
            var property = _.checkCalibration(options.properties, options.time, this.config.enableCalibrationTime)

            var time = _.isDate(options.time) ? options.time : new Date()

            this._sendRequest(
              {
                type: 'track',
                eventName: options.eventName,
                properties: property,
                onComplete: options.onComplete,
                firstCheckId: options.firstCheckId ? options.firstCheckId : this.getDeviceId(),
              },
              time
            )
          } else {
            //options.time = time;
            this._queue.push(['trackFirstEvent', [options]])
          }
        } else {
          logger$1.warn('Invalide parameter for trackFirstEvent: you should pass an object contains eventName to trackFirstEvent()')

          if (_.isFunction(options.onComplete)) {
            options.onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      }, // internal function. Do not call this function directly.
    },
    {
      key: '_internalTrack',
      value: function _internalTrack(eventName, properties, time, onComplete, tryBeacon) {
        if (this._hasDisabled()) {
          return
        }

        var property = _.checkCalibration(properties, time, this.config.enableCalibrationTime)

        time = _.isDate(time) ? time : new Date()

        if (this._isReady()) {
          this._sendRequest(
            {
              type: 'track',
              eventName: eventName,
              properties: property,
              onComplete: onComplete,
            },
            time,
            tryBeacon
          )
        } else {
          this._queue.push(['_internalTrack', [eventName, properties, time, onComplete]])
        }
      },
      /**
       * Sets the user property, replacing the original value with the new value if the property already exists.
       * @param {*} properties event properties, optional
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userSet',
      value: function userSet(properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.propertiesMust(properties) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            this._sendRequest(
              {
                type: 'user_set',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userSet', [properties, time, onComplete]])
          }
        } else {
          logger$1.warn('calling userSet failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Sets a single user attribute, ignoring the new attribute value if the attribute already exists.
       * @param {*} properties event properties, optional
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userSetOnce',
      value: function userSetOnce(properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.propertiesMust(properties) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            this._sendRequest(
              {
                type: 'user_setOnce',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userSetOnce', [properties, time, onComplete]])
          }
        } else {
          logger$1.warn('calling userSetOnce failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Reset user properties.
       * @param {*} property event property, optional
       * @param {*} time  event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userUnset',
      value: function userUnset(property, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          property = options.property
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.propertyName(property) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            var properties = {}
            properties[property] = 0

            this._sendRequest(
              {
                type: 'user_unset',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userUnset', [property, onComplete, time]])
          }
        } else {
          logger$1.warn('calling userUnset failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Delete the user attributes,This operation is not reversible and should be performed with caution.
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userDel',
      value: function userDel(time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(time)) {
          var options = time
          time = options.time
          onComplete = options.onComplete
        }

        time = _.isDate(time) ? time : new Date()

        if (this._isReady()) {
          this._sendRequest(
            {
              type: 'user_del',
              onComplete: onComplete,
            },
            time
          )
        } else {
          this._queue.push(['userDel', [time, onComplete]])
        }
      },
      /**
       * Adds the numeric type user attributes.
       * @param {*} properties event properties, optional
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userAdd',
      value: function userAdd(properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.userAddProperties(properties) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            this._sendRequest(
              {
                type: 'user_add',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userAdd', [properties, time, onComplete]])
          }
        } else {
          logger$1.warn('calling userAdd failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Append a user attribute of the List type.
       * @param {*} properties event properties, optional
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userAppend',
      value: function userAppend(properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.userAppendProperties(properties) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            this._sendRequest(
              {
                type: 'user_append',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userAppend', [properties, time, onComplete]])
          }
        } else {
          logger$1.warn('calling userAppend failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * The element appended to the library needs to be done to remove the processing,and then import.
       * @param {*} properties event properties, optional
       * @param {*} time event time, optional
       * @param {*} onComplete callback, optional
       * @returns
       */
    },
    {
      key: 'userUniqAppend',
      value: function userUniqAppend(properties, time, onComplete) {
        if (this._hasDisabled()) {
          return
        }

        if (this._isObjectParams(properties)) {
          var options = properties
          properties = options.properties
          time = options.time
          onComplete = options.onComplete
        }

        if (PropertyChecker.userAppendProperties(properties) || !this.config.strict) {
          time = _.isDate(time) ? time : new Date()

          if (this._isReady()) {
            this._sendRequest(
              {
                type: 'user_uniq_append',
                properties: properties,
                onComplete: onComplete,
              },
              time
            )
          } else {
            this._queue.push(['userUniqAppend', [properties, time, onComplete]])
          }
        } else {
          logger$1.warn('calling userAppend failed due to invalid arguments')

          if (_.isFunction(onComplete)) {
            onComplete({
              code: -1,
              msg: 'invalid parameters',
            })
          }
        }
      },
      /**
       * Empty the cache queue. When this api is called, the data in the current cache queue will attempt to be reported.
       * If the report succeeds, local cache data will be deleted.
       */
    },
    {
      key: 'flush',
      value: function flush() {
        if (this.batchConsumer && this.config.debugMode === 'none') {
          this.batchConsumer.flush()
        }
      },
    },
    {
      key: 'authorizeOpenID',
      value: function authorizeOpenID(id) {
        this.identify(id)
      },
      /**
       * Set the distinct ID to replace the default UUID distinct ID.
       * @param {*} distinctId distinct ID
       * @returns
       */
    },
    {
      key: 'identify',
      value: function identify(distinctId) {
        if (this._hasDisabled()) {
          return
        }

        if (typeof distinctId === 'number') {
          distinctId = String(distinctId)
        } else if (typeof distinctId !== 'string') {
          return false
        }

        this.store.setDistinctId(distinctId)
        logger$1.info('Setting distinct ID, DistinctId = ' + distinctId)
      },
      /**
       * Get a visitor ID: The #distinct_id value in the reported data.
       * @returns distinct ID
       */
    },
    {
      key: 'getDistinctId',
      value: function getDistinctId() {
        return this.store.getDistinctId()
      },
      /**
       * Set the account ID. Each setting overrides the previous value. Login events will not be uploaded.
       * @param {*} accoundId
       * @returns
       */
    },
    {
      key: 'login',
      value: function login(accoundId) {
        if (this._hasDisabled()) {
          return
        }

        if (typeof accoundId === 'number') {
          accoundId = String(accoundId)
        } else if (typeof accoundId !== 'string') {
          return false
        }

        this.store.setAccountId(accoundId)
        logger$1.info('Login SDK, AccountId = ' + accoundId)
      },
    },
    {
      key: 'getAccountId',
      value: function getAccountId() {
        return this.store.getAccountId()
      },
      /**
       * Clearing the account ID will not upload user logout events.
       * @returns
       */
    },
    {
      key: 'logout',
      value: function logout() {
        if (this._hasDisabled()) {
          return
        }

        this.store.setAccountId(null)
        logger$1.info('Logout SDK')
      },
      /**
       * Set the public event attribute, which will be included in every event uploaded after that. The public event properties are saved without setting them each time.
       * @param {*} obj public event attribute
       * @returns
       */
    },
    {
      key: 'setSuperProperties',
      value: function setSuperProperties(obj) {
        if (this._hasDisabled()) {
          return
        }

        if (PropertyChecker.propertiesMust(obj) || !this.config.strict) {
          this.store.setSuperProperties(obj)
        } else {
          logger$1.warn('setSuperProperties parameter must be a valid property value')
        }
      },
      /**
       * Clear all public event attributes.
       * @returns
       */
    },
    {
      key: 'clearSuperProperties',
      value: function clearSuperProperties() {
        if (this._hasDisabled()) {
          return
        }

        this.store.setSuperProperties({}, true)
      },
      /**
       * Clears a public event attribute.
       * @param {*} propertyName Public event attribute key to clear
       * @returns
       */
    },
    {
      key: 'unsetSuperProperty',
      value: function unsetSuperProperty(propertyName) {
        if (this._hasDisabled()) {
          return
        }

        if (_.isString(propertyName)) {
          var superProperties = this.getSuperProperties()
          delete superProperties[propertyName]
          this.store.setSuperProperties(superProperties, true)
        }
      },
      /**
       * Gets the public event properties that have been set.
       * @returns Public event properties that have been set
       */
    },
    {
      key: 'getSuperProperties',
      value: function getSuperProperties() {
        return this.store.getSuperProperties()
      },
      /**
       * Gets prefabricated properties for all events.
       * @returns
       */
    },
    {
      key: 'getPresetProperties',
      value: function getPresetProperties() {
        var properties = systemInformation.properties
        var presetProperties = {}
        var os = properties['#os']
        presetProperties.os = _.isUndefined(os) ? '' : os
        var screenWidth = properties['#screen_width']
        presetProperties.screenWidth = _.isUndefined(screenWidth) ? 0 : screenWidth
        var screenHeight = properties['#screen_height']
        presetProperties.screenHeight = _.isUndefined(screenHeight) ? 0 : screenHeight
        var networkType = properties['#network_type']
        presetProperties.networkType = _.isUndefined(networkType) ? '' : networkType
        var deviceModel = properties['#device_model']
        presetProperties.deviceModel = _.isUndefined(deviceModel) ? '' : deviceModel
        var osVersion = properties['#os_version']
        presetProperties.osVersion = _.isUndefined(osVersion) ? '' : osVersion
        presetProperties.deviceId = this.getDeviceId()
        var zoneOffset = 0 - new Date().getTimezoneOffset() / 60.0
        presetProperties.zoneOffset = zoneOffset
        var manufacturer = properties['#manufacturer']
        presetProperties.manufacturer = _.isUndefined(manufacturer) ? '' : manufacturer

        presetProperties.toEventPresetProperties = function () {
          return {
            '#device_model': presetProperties.deviceModel,
            '#device_id': presetProperties.deviceId,
            '#screen_width': presetProperties.screenWidth,
            '#screen_height': presetProperties.screenHeight,
            '#os': presetProperties.os,
            '#os_version': presetProperties.osVersion,
            '#network_type': presetProperties.networkType,
            '#zone_offset': zoneOffset,
            '#manufacturer': presetProperties.manufacturer,
          }
        }

        return presetProperties
      },
      /**
       * Set dynamic public properties. Each event uploaded after that will contain a public event attribute.
       * @param {*} dynamicProperties Dynamic public attribute interface
       * @returns
       */
    },
    {
      key: 'setDynamicSuperProperties',
      value: function setDynamicSuperProperties(dynamicProperties) {
        if (this._hasDisabled()) {
          return
        }

        if (typeof dynamicProperties === 'function') {
          if (PropertyChecker.properties(dynamicProperties()) || !this.config.strict) {
            this.dynamicProperties = dynamicProperties
          } else {
            logger$1.warn('A dynamic public property must return a valid property value')
          }
        } else {
          logger$1.warn('setDynamicSuperProperties parameter must be a function type')
        }
      },
      /**
       * Record the event duration, call this method to start the timing, stop the timing when the target event is uploaded, and add the attribute #duration to the event properties, in seconds.
       * @param {*} eventName target event name
       * @param {*} time
       * @returns
       */
    },
    {
      key: 'timeEvent',
      value: function timeEvent(eventName, time) {
        if (this._hasDisabled()) {
          return
        }

        time = _.isDate(time) ? time : new Date()

        if (this._isReady()) {
          if (PropertyChecker.event(eventName) || !this.config.strict) {
            this.store.setEventTimer(eventName, time.getTime())
          } else {
            logger$1.warn('calling timeEvent failed due to invalid eventName: ' + eventName)
          }
        } else {
          this._queue.push(['timeEvent', [eventName, time]])
        }
      },
    },
    {
      key: 'getDeviceId',
      value: function getDeviceId() {
        return systemInformation.properties['#device_id']
      },
      /**
       * Pause/Resume reporting event data
       * @param {bool} enabled:true is Resume, false is Pause
       * @deprecated This method is deprecated, use setTrackStatus() instand.
       */
    },
    {
      key: 'enableTracking',
      value: function enableTracking(enabled) {
        this.enabled = enabled

        this.store._set('ta_enabled', enabled)
      },
      /**
       * Stop reporting event data, and cache data will be cleared
       * @deprecated This method is deprecated, use setTrackStatus() instand.
       */
    },
    {
      key: 'optOutTracking',
      value: function optOutTracking() {
        this.store.setSuperProperties({}, true)
        this.store.setDistinctId(_.UUID())
        this.store.setAccountId(null)

        this._queue.splice(0, this._queue.length)

        this.isOptOut = true

        this.store._set('ta_isOptOut', true)
      },
      /**
       * Stop reporting event data, and cache data will be cleared, and flush a user_del
       * @deprecated This method is deprecated, use setTrackStatus() instand.
       */
    },
    {
      key: 'optOutTrackingAndDeleteUser',
      value: function optOutTrackingAndDeleteUser() {
        var time = new Date()

        this._sendRequest(
          {
            type: 'user_del',
          },
          time
        )

        this.optOutTracking()
      },
      /**
       * Allow reporting event data
       * @deprecated This method is deprecated, use setTrackStatus() instand.
       */
    },
    {
      key: 'optInTracking',
      value: function optInTracking() {
        this.isOptOut = false

        this.store._set('ta_isOptOut', false)
      },
      /**
       * Set status for events reporting
       * PAUSE, pause events reporting
       * STOP, stop events reporting, and cache data will be cleared
       * SAVE_ONLY, event data stores in the cache, but not be reported (native support, js equal to NORMAL)
       * NORMAL, resume event reporting
       * @param {string} status, events reporting status
       */
    },
    {
      key: 'setTrackStatus',
      value: function setTrackStatus(status) {
        switch (status) {
          case 'PAUSE':
            this.eventSaveOnly = false
            this.optInTracking()
            this.enableTracking(false)
            break

          case 'STOP':
            this.eventSaveOnly = false // this.enableTracking(false);

            this.optOutTracking(true)
            break

          case 'SAVE_ONLY':
            // this.eventSaveOnly = true;
            // this.enableTracking(false);
            // this.optInTracking();
            break

          case 'NORMAL':
          default:
            this.eventSaveOnly = false
            this.optInTracking()
            this.enableTracking(true)
            break
        }

        logger$1.info('Change Status to ' + status)
      },
    },
  ])

  return ThinkingDataAPI
})()

var DEFAULT_CONFIG$1 = {
  name: 'thinkingdata',
  // Global variable name
  enableLog: true,
  // Whether to enable log
  enableNative: false,
}

var ThinkingAnalyticsAPI = /*#__PURE__*/ (function () {
  /**
   * Create an sdk instance and pass in configuration information
   * @param {*} config configuration information
   */
  function ThinkingAnalyticsAPI(config) {
    _classCallCheck(this, ThinkingAnalyticsAPI)

    config.appId = config.appId ? _.checkAppId(config.appId) : _.checkAppId(config.appid)
    config.serverUrl = config.serverUrl ? _.checkUrl(config.serverUrl) : _.checkUrl(config.server_url)

    var defaultConfig = _.extend({}, DEFAULT_CONFIG$1, PlatformAPI.getConfig())

    if (_.isObject(config)) {
      this.config = _.extend(defaultConfig, config)
    } else {
      this.config = defaultConfig
    }

    this._init(this.config)
  }

  _createClass(ThinkingAnalyticsAPI, [
    {
      key: '_isNativePlatform',
      value: function _isNativePlatform() {
        // #ifdef  APP-PLUS
        return true // #endif
      },
    },
    {
      key: '_isIOS',
      value: function _isIOS() {
        if (_.isUndefined(this.isiOSPlat)) {
          var iOSApiClass = plus.ios.importClass('ThinkingGameEngineApi')

          if (iOSApiClass) {
            this.isiOSPlat = true
          } else {
            this.isiOSPlat = false
          }
        }

        return this.isiOSPlat
      },
    },
    {
      key: '_isAndroid',
      value: function _isAndroid() {
        if (_.isUndefined(this.isAndroidPlat)) {
          var androidApiClass = plus.android.importClass('cn.thinkingdata.engine.ThinkingAnalyticsUniAppAPI')

          if (androidApiClass) {
            this.isAndroidPlat = true
          } else {
            this.isAndroidPlat = false
          }
        }

        return this.isAndroidPlat
      },
    },
    {
      key: '_init',
      value: function _init(config) {
        this.name = config.name
        this.appId = config.appId || config.appid

        if (this._isNativePlatform()) {
          this.initInstanceForNative() //this._readStorage(config);
        } else {
          this.taJs = new ThinkingDataAPI(config)
        }
      },
      /**
       * SDK initialization
       * @returns sdk instance
       */
    },
    {
      key: 'init',
      value: function init() {
        if (this._isNativePlatform()) {
          this.startThinkingAnalyticsForNative()
          return
        }

        this.taJs.init()
      },
    },
    {
      key: 'startThinkingAnalyticsForNative',
      value: function startThinkingAnalyticsForNative() {
        var mode = 0

        if (this.config.debugMode === 'debug') {
          mode = 1
        } else if (this.config.debugMode === 'debugOnly') {
          mode = 2
        }

        var secretKey = {}

        if (_.isObject(this.config.secretKey)) {
          secretKey.publicKey = this.config.secretKey.publicKey
          secretKey.version = this.config.secretKey.version
          secretKey.symmetricEncryption = 'AES'
          secretKey.asymmetricEncryption = 'RSA'
        }

        var tConfig = {
          appId: this.config.appId,
          serverUrl: this.config.serverUrl,
          mode: mode,
          enableEncrypt: this.config.enableEncrypt,
          secretKey: secretKey,
          timeZone: this.config.timeZone,
        }
        var autoTrack = []

        if (_.isObject(this.config.autoTrack)) {
          if (this.config.autoTrack.appInstall) {
            autoTrack.push('appInstall')
          }

          if (this.config.autoTrack.appShow) {
            autoTrack.push('appStart')
          }

          if (this.config.autoTrack.appHide) {
            autoTrack.push('appEnd')
          }

          if (this.config.autoTrack.appCrash) {
            autoTrack.push('appCrash')
          }
        }

        var str = {
          appId: this.appId,
          autoTrack: autoTrack,
        }

        if (this._isAndroid()) {
          this.androidApi.setCustomerLibInfo(Config.LIB_NAME, Config.LIB_VERSION)
          this.androidApi.enableTrackLog(this.config.enableLog)
          this.androidApi.sharedInstance(JSON.stringify(tConfig))
          this.androidApi.enableAutoTrack(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.setCustomerLibInfolibVersion(Config.LIB_NAME, Config.LIB_VERSION)
          this.iosApi.enableTrackLog(this.config.enableLog)
          this.iosApi.sharedInstance(JSON.stringify(tConfig))
          this.iosApi.enableAutoTrack(JSON.stringify(str))
        }
      },
    },
    {
      key: 'initInstance',
      value: function initInstance(name, config) {
        if (this._isNativePlatform()) {
          if (!_.isUndefined(config)) {
            this[name] = new ThinkingAnalyticsAPI(config)
          } else {
            this[name] = new ThinkingAnalyticsAPI(this.config)
          }

          this[name].init()
          return this[name]
        }

        this[name] = this.taJs.initInstance(name, config)
        return this[name]
      },
    },
    {
      key: 'lightInstance',
      value: function lightInstance(name) {
        return this[name]
      },
    },
    {
      key: 'initInstanceForNative',
      value: function initInstanceForNative() {
        if (this._isAndroid()) {
          var androidApiClass = plus.android.importClass('cn.thinkingdata.engine.ThinkingAnalyticsUniAppAPI')
          this.androidApi = new androidApiClass()
        } else if (this._isIOS()) {
          var iOSApiClass = plus.ios.importClass('ThinkingGameEngineApi')
          this.iosApi = new iOSApiClass()
        }
      },
    },
    {
      key: 'calibrateTime',
      value: function calibrateTime(timeStampMillis) {
        // #ifdef  APP-PLUS
        if (this._isAndroid()) {
          this.androidApi.calibrateTime(timeStampMillis)
        } else if (this._isIOS()) {
          this.iosApi.calibrateTime(timeStampMillis)
        } // #endif
      },
    },
    {
      key: 'calibrateTimeWithNtp',
      value: function calibrateTimeWithNtp(ntpServer) {
        // #ifdef  APP-PLUS
        if (this._isAndroid()) {
          this.androidApi.calibrateTimeWithNtp(ntpServer)
        } else if (this._isIOS()) {
          this.iosApi.calibrateTimeWithNtp(ntpServer)
        } // #endif
      },
      /**
       * track a event
       * @param {*} eventName event name
       * @param {*} properties  event properties
       * @param {*} time event time
       * @param {*} onComplete callback
       */
    },
    {
      key: 'track',
      value: function track(eventName, properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.trackForNative(eventName, properties, time)
          return
        }

        this.taJs.track(eventName, properties, time, onComplete)
      },
    },
    {
      key: 'trackForNative',
      value: function trackForNative(eventName, properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        if (_.isUndefined(properties)) properties = {}
        properties = _.extend(properties, this.dynamicProperties ? this.dynamicProperties() : {})
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          eventName: eventName,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.track(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.track(JSON.stringify(str))
        }
      },
      /**
       * You can implement the requirement to modify event data in a specific scenario through updatable events.
       * Updatable events need to specify an ID that identifies the event and pass it in when the updatable event object is created.
       * @param {*} options
       */
    },
    {
      key: 'trackUpdate',
      value: function trackUpdate(options) {
        if (this._isNativePlatform()) {
          this.trackUpdateForNative(options)
          return
        }

        this.taJs.trackUpdate(options)
      },
    },
    {
      key: 'trackUpdateForNative',
      value: function trackUpdateForNative(options) {
        var properties = _.extend(!_.isUndefined(options.properties) ? options.properties : {}, this.dynamicProperties ? this.dynamicProperties() : {})

        properties = _.encodeDates(properties)
        var timeStamp = _.isDate(options.time) ? options.time.getTime() : ''
        var str = {
          appId: this.appId,
          eventName: options.eventName,
          properties: properties,
          type: 1,
          eventId: options.eventId,
          time: timeStamp,
          timeZone: options.timeZone,
        }

        if (this._isAndroid()) {
          this.androidApi.trackEvent(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.trackEvent(JSON.stringify(str))
        }
      },
      /**
       * Rewritable events will completely cover historical data with the latest data,
       * which is equivalent to deleting the previous data and storing the latest data in effect.
       * @param {*} options
       */
    },
    {
      key: 'trackOverwrite',
      value: function trackOverwrite(options) {
        if (this._isNativePlatform()) {
          this.trackOverwriteForNative(options)
          return
        }

        this.taJs.trackOverwrite(options)
      },
    },
    {
      key: 'trackOverwriteForNative',
      value: function trackOverwriteForNative(options) {
        var properties = _.extend(!_.isUndefined(options.properties) ? options.properties : {}, this.dynamicProperties ? this.dynamicProperties() : {})

        properties = _.encodeDates(properties)
        var timeStamp = _.isDate(options.time) ? options.time.getTime() : ''
        var str = {
          appId: this.appId,
          eventName: options.eventName,
          properties: properties,
          type: 2,
          eventId: options.eventId,
          time: timeStamp,
          timeZone: options.timeZone,
        }

        if (this._isAndroid()) {
          this.androidApi.trackEvent(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.trackEvent(JSON.stringify(str))
        }
      },
      /**
       * The first event refers to the ID of a device or other dimension, which will only be recorded once.
       * @param {*} options
       */
    },
    {
      key: 'trackFirstEvent',
      value: function trackFirstEvent(options) {
        if (this._isNativePlatform()) {
          this.trackFirstEventForNative(options)
          return
        }

        this.taJs.trackFirstEvent(options)
      },
    },
    {
      key: 'trackFirstEventForNative',
      value: function trackFirstEventForNative(options) {
        var properties = _.extend(!_.isUndefined(options.properties) ? options.properties : {}, this.dynamicProperties ? this.dynamicProperties() : {})

        properties = _.encodeDates(properties)
        var timeStamp = _.isDate(options.time) ? options.time.getTime() : ''
        var str = {
          appId: this.appId,
          eventName: options.eventName,
          properties: properties,
          type: 0,
          eventId: options.firstCheckId,
          time: timeStamp,
          timeZone: options.timeZone,
        }

        if (this._isAndroid()) {
          this.androidApi.trackEvent(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.trackEvent(JSON.stringify(str))
        }
      },
      /**
       * Sets the user property, replacing the original value with the new value if the property already exists.
       * @param {*} properties user properties
       * @param {*} time track time
       * @param {*} onComplete callback
       */
    },
    {
      key: 'userSet',
      value: function userSet(properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userSetForNative(properties, time)
          return
        }

        this.taJs.userSet(properties, time, onComplete)
      },
    },
    {
      key: 'userSetForNative',
      value: function userSetForNative(properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userSet(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userSet(JSON.stringify(str))
        }
      },
      /**
       * Sets a single user attribute, ignoring the new attribute value if the attribute already exists.
       * @param {*} properties user properties
       * @param {*} time track time
       * @param {*} onComplete callback
       * @returns
       */
    },
    {
      key: 'userSetOnce',
      value: function userSetOnce(properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userSetOnceForNative(properties, time)
          return
        }

        this.taJs.userSetOnce(properties, time, onComplete)
      },
    },
    {
      key: 'userSetOnceForNative',
      value: function userSetOnceForNative(properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userSetOnce(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userSetOnce(JSON.stringify(str))
        }
      },
      /**
       * Reset user properties.
       * @param {*} property
       * @param {*} time
       * @param {*} onComplete
       * @returns
       */
    },
    {
      key: 'userUnset',
      value: function userUnset(property, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userUnsetForNative(property, time)
          return
        }

        this.taJs.userUnset(property, time, onComplete)
      },
    },
    {
      key: 'userUnsetForNative',
      value: function userUnsetForNative(property, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        var str = {
          appId: this.appId,
          properties: [property],
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userUnset(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userUnset(JSON.stringify(str))
        }
      },
      /**
       * Delete the user attributes,This operation is not reversible and should be performed with caution.
       * @param {*} time
       * @param {*} onComplete
       * @returns
       */
    },
    {
      key: 'userDel',
      value: function userDel(time, onComplete) {
        if (this._isNativePlatform()) {
          this.userDelForNative(time)
          return
        }

        this.taJs.userDel(time, onComplete)
      },
    },
    {
      key: 'userDelForNative',
      value: function userDelForNative(time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        var str = {
          appId: this.appId,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userDel(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userDel(JSON.stringify(str))
        }
      },
      /**
       * Adds the numeric type user attributes.
       * @param {*} properties
       * @param {*} time
       * @param {*} onComplete
       * @returns
       */
    },
    {
      key: 'userAdd',
      value: function userAdd(properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userAddForNative(properties, time)
          return
        }

        this.taJs.userAdd(properties, time, onComplete)
      },
    },
    {
      key: 'userAddForNative',
      value: function userAddForNative(properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userAdd(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userAdd(JSON.stringify(str))
        }
      },
      /**
       * Append a user attribute of the List type.
       * @param {*} properties
       * @param {*} time
       * @param {*} onComplete
       * @returns
       */
    },
    {
      key: 'userAppend',
      value: function userAppend(properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userAppendForNative(properties, time)
          return
        }

        this.taJs.userAppend(properties, time, onComplete)
      },
    },
    {
      key: 'userAppendForNative',
      value: function userAppendForNative(properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userAppend(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userAppend(JSON.stringify(str))
        }
      },
      /**
       * The element appended to the library needs to be done to remove the processing,and then import.
       * @param {*} properties
       * @param {*} time
       * @param {*} onComplete
       * @returns
       */
    },
    {
      key: 'userUniqAppend',
      value: function userUniqAppend(properties, time, onComplete) {
        if (this._isNativePlatform()) {
          this.userUniqAppendForNative(properties, time)
          return
        }

        this.taJs.userUniqAppend(properties, time, onComplete)
      },
    },
    {
      key: 'userUniqAppendForNative',
      value: function userUniqAppendForNative(properties, time) {
        var timeStamp = _.isDate(time) ? time.getTime() : ''
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
          time: timeStamp,
        }

        if (this._isAndroid()) {
          this.androidApi.userUniqAppend(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.userUniqAppend(JSON.stringify(str))
        }
      },
      /**
       * Empty the cache queue. When this api is called, the data in the current cache queue will attempt to be reported.
       * If the report succeeds, local cache data will be deleted.
       * @returns
       */
    },
    {
      key: 'flush',
      value: function flush() {
        if (this._isNativePlatform()) {
          this.flushForNative()
          return
        }

        this.taJs.flush()
      },
    },
    {
      key: 'flushForNative',
      value: function flushForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          this.androidApi.flush(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.flush(JSON.stringify(str))
        }
      },
    },
    {
      key: 'authorizeOpenID',
      value: function authorizeOpenID(id) {
        this.identify(id)
      },
      /**
       * Set the distinct ID to replace the default UUID distinct ID.
       * @param {*} id
       * @returns
       */
    },
    {
      key: 'identify',
      value: function identify(id) {
        if (this._isNativePlatform()) {
          this.identifyForNative(id)
          return
        }

        this.taJs.identify(id)
      },
    },
    {
      key: 'identifyForNative',
      value: function identifyForNative(distinctId) {
        var str = {
          appId: this.appId,
          distinctId: distinctId,
        }

        if (this._isAndroid()) {
          this.androidApi.identify(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.identify(JSON.stringify(str))
        }
      },
      /**
       * Get a visitor ID: The #distinct_id value in the reported data.
       * @returns distinct ID
       */
    },
    {
      key: 'getDistinctId',
      value: function getDistinctId() {
        if (this._isNativePlatform()) {
          return this.getDistinctIdForNative()
        }

        return this.taJs.getDistinctId()
      },
    },
    {
      key: 'getDistinctIdForNative',
      value: function getDistinctIdForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          return this.androidApi.getDistinctId(JSON.stringify(str))
        } else if (this._isIOS()) {
          return this.iosApi.getDistinctId(JSON.stringify(str))
        }
      },
      /**
       * Set the account ID. Each setting overrides the previous value. Login events will not be uploaded.
       * @param {*} accoundId
       * @returns
       */
    },
    {
      key: 'login',
      value: function login(accoundId) {
        if (this._isNativePlatform()) {
          this.loginForNative(accoundId)
          return
        }

        this.taJs.login(accoundId)
      },
    },
    {
      key: 'loginForNative',
      value: function loginForNative(accoundId) {
        var str = {
          appId: this.appId,
          loginId: accoundId,
        }

        if (this._isAndroid()) {
          this.androidApi.login(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.login(JSON.stringify(str))
        }
      },
    },
    {
      key: 'getAccountId',
      value: function getAccountId() {
        if (this._isNativePlatform()) {
          return this.getAccountIdForNative()
        }

        return this.taJs.getAccountId()
      },
    },
    {
      key: 'getAccountIdForNative',
      value: function getAccountIdForNative() {
        return ''
      },
      /**
       * Clearing the account ID will not upload user logout events.
       * @returns
       */
    },
    {
      key: 'logout',
      value: function logout() {
        if (this._isNativePlatform()) {
          this.logoutForNative()
          return
        }

        this.taJs.logout()
      },
    },
    {
      key: 'logoutForNative',
      value: function logoutForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          this.androidApi.logout(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.logout(JSON.stringify(str))
        }
      },
      /**
       * Set the public event attribute, which will be included in every event uploaded after that.
       * The public event properties are saved without setting them each time.
       * @param {*} obj
       * @returns
       */
    },
    {
      key: 'setSuperProperties',
      value: function setSuperProperties(obj) {
        if (this._isNativePlatform()) {
          this.setSuperPropertiesForNative(obj)
          return
        }

        this.taJs.setSuperProperties(obj)
      },
    },
    {
      key: 'setSuperPropertiesForNative',
      value: function setSuperPropertiesForNative(properties) {
        properties = _.encodeDates(properties)
        var str = {
          appId: this.appId,
          properties: properties,
        }

        if (this._isAndroid()) {
          this.androidApi.setSuperProperties(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.setSuperProperties(JSON.stringify(str))
        }
      },
      /**
       * Clear all public event attributes.
       * @returns
       */
    },
    {
      key: 'clearSuperProperties',
      value: function clearSuperProperties() {
        if (this._isNativePlatform()) {
          this.clearSuperPropertiesForNative()
          return
        }

        this.taJs.clearSuperProperties()
      },
    },
    {
      key: 'clearSuperPropertiesForNative',
      value: function clearSuperPropertiesForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          this.androidApi.clearSuperProperties(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.clearSuperProperties(JSON.stringify(str))
        }
      },
      /**
       * Clears a public event attribute.
       * @param {*} propertyName
       * @returns
       */
    },
    {
      key: 'unsetSuperProperty',
      value: function unsetSuperProperty(propertyName) {
        if (this._isNativePlatform()) {
          this.unsetSuperPropertyForNative(propertyName)
          return
        }

        this.taJs.unsetSuperProperty(propertyName)
      },
    },
    {
      key: 'unsetSuperPropertyForNative',
      value: function unsetSuperPropertyForNative(propertyName) {
        var str = {
          appId: this.appId,
          property: propertyName,
        }

        if (this._isAndroid()) {
          this.androidApi.unsetSuperProperty(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.unsetSuperProperty(JSON.stringify(str))
        }
      },
      /**
       * Gets the public event properties that have been set.
       * @returns Public event properties that have been set
       */
    },
    {
      key: 'getSuperProperties',
      value: function getSuperProperties() {
        if (this._isNativePlatform()) {
          return this.getSuperPropertiesForNative()
        }

        return this.taJs.getSuperProperties()
      },
    },
    {
      key: 'getSuperPropertiesForNative',
      value: function getSuperPropertiesForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          return this.androidApi.getSuperProperties(JSON.stringify(str))
        } else if (this._isIOS()) {
          return this.iosApi.getSuperProperties(JSON.stringify(str))
        }
      },
      /**
       * Gets prefabricated properties for all events.
       * @returns
       */
    },
    {
      key: 'getPresetProperties',
      value: function getPresetProperties() {
        if (this._isNativePlatform()) {
          var properties = JSON.parse(this.getPresetPropertiesForNative())
          var presetProperties = {}
          var os = properties['#os']
          presetProperties.os = _.isUndefined(os) ? '' : os
          var screenWidth = properties['#screen_width']
          presetProperties.screenWidth = _.isUndefined(screenWidth) ? 0 : screenWidth
          var screenHeight = properties['#screen_height']
          presetProperties.screenHeight = _.isUndefined(screenHeight) ? 0 : screenHeight
          var networkType = properties['#network_type']
          presetProperties.networkType = _.isUndefined(networkType) ? '' : networkType
          var deviceModel = properties['#device_model']
          presetProperties.deviceModel = _.isUndefined(deviceModel) ? '' : deviceModel
          var osVersion = properties['#os_version']
          presetProperties.osVersion = _.isUndefined(osVersion) ? '' : osVersion
          presetProperties.deviceId = this.getDeviceId()
          var zoneOffset = properties['#zone_offset']
          presetProperties.zoneOffset = _.isUndefined(zoneOffset) ? '' : zoneOffset
          var manufacturer = properties['#manufacturer']
          presetProperties.manufacturer = _.isUndefined(manufacturer) ? '' : manufacturer

          presetProperties.toEventPresetProperties = function () {
            return {
              '#device_model': presetProperties.deviceModel,
              '#device_id': presetProperties.deviceId,
              '#screen_width': presetProperties.screenWidth,
              '#screen_height': presetProperties.screenHeight,
              '#os': presetProperties.os,
              '#os_version': presetProperties.osVersion,
              '#network_type': presetProperties.networkType,
              '#zone_offset': zoneOffset,
              '#manufacturer': presetProperties.manufacturer,
            }
          }

          return presetProperties
        }

        return this.taJs.getPresetProperties()
      },
    },
    {
      key: 'getPresetPropertiesForNative',
      value: function getPresetPropertiesForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          return this.androidApi.getPresetProperties(JSON.stringify(str))
        } else if (this._isIOS()) {
          return this.iosApi.getPresetProperties(JSON.stringify(str))
        }
      },
      /**
       * Set dynamic public properties. Each event uploaded after that will contain a public event attribute.
       * @param {*} dynamicProperties
       * @returns
       */
    },
    {
      key: 'setDynamicSuperProperties',
      value: function setDynamicSuperProperties(dynamicProperties) {
        if (this._isNativePlatform()) {
          if (typeof dynamicProperties === 'function') {
            this.dynamicProperties = dynamicProperties
          } else {
            logger.warn('setDynamicSuperProperties parameter must be a function type')
          }

          return
        }

        this.taJs.setDynamicSuperProperties(dynamicProperties)
      },
      /**
       * Record the event duration, call this method to start the timing, stop the timing when the target event is uploaded,
       * and add the attribute #duration to the event properties, in seconds.
       * @param {*} eventName
       * @param {*} time
       * @returns
       */
    },
    {
      key: 'timeEvent',
      value: function timeEvent(eventName, time) {
        if (this._isNativePlatform()) {
          this.timeEventForNative(eventName)
          return
        }

        this.taJs.timeEvent(eventName, time)
      },
    },
    {
      key: 'timeEventForNative',
      value: function timeEventForNative(eventName) {
        var str = {
          appId: this.appId,
          eventName: eventName,
        }

        if (this._isAndroid()) {
          this.androidApi.timeEvent(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.timeEvent(JSON.stringify(str))
        }
      },
    },
    {
      key: 'getDeviceId',
      value: function getDeviceId() {
        if (this._isNativePlatform()) {
          return this.getDeviceIdForNative()
        }

        return this.taJs.getDeviceId()
      },
    },
    {
      key: 'getDeviceIdForNative',
      value: function getDeviceIdForNative() {
        var str = {
          appId: this.appId,
        }

        if (this._isAndroid()) {
          return this.androidApi.getDeviceId(JSON.stringify(str))
        } else if (this._isIOS()) {
          return this.iosApi.getDeviceId(JSON.stringify(str))
        }
      },
    },
    {
      key: 'enableTracking',
      value: function enableTracking(enabled) {
        if (this._isNativePlatform()) {
          if (enabled) {
            this.setTrackStatus('NORMAL')
          } else {
            this.setTrackStatus('PAUSE')
          }

          return
        }

        this.taJs.enableTracking(enabled)
      },
    },
    {
      key: 'optOutTracking',
      value: function optOutTracking() {
        if (this._isNativePlatform()) {
          this.setTrackStatus('STOP')
          return
        }

        this.taJs.optOutTracking()
      },
    },
    {
      key: 'optOutTrackingAndDeleteUser',
      value: function optOutTrackingAndDeleteUser() {
        if (this._isNativePlatform()) {
          return
        }

        this.taJs.optOutTrackingAndDeleteUser()
      },
    },
    {
      key: 'optInTracking',
      value: function optInTracking() {
        if (this._isNativePlatform()) {
          this.setTrackStatus('NORMAL')
          return
        }

        this.taJs.optInTracking()
      },
      /**
       * Set the data reporting status
       * PAUSE Suspending Data Reporting
       * STOP Stop data reporting and clear the cache
       * SAVE_ONLY Data is stored, but not reported (support native)
       * NORMAL Reporting Restored Data
       * @param {string} status
       */
    },
    {
      key: 'setTrackStatus',
      value: function setTrackStatus(status) {
        if (this._isNativePlatform()) {
          this.setTrackStatusForNative(status)
          return
        }

        this.taJs.setTrackStatus(status)
      },
    },
    {
      key: 'setTrackStatusForNative',
      value: function setTrackStatusForNative(status) {
        var sta = 'normal'

        switch (status) {
          case 'PAUSE':
            sta = 'pause'
            break

          case 'STOP':
            sta = 'stop'
            break

          case 'SAVE_ONLY':
            sta = 'saveOnly'
            break

          case 'NORMAL':
          default:
            sta = 'normal'
            break
        }

        var str = {
          appId: this.appId,
          status: sta,
        }

        if (this._isAndroid()) {
          this.androidApi.setTrackStatus(JSON.stringify(str))
        } else if (this._isIOS()) {
          this.iosApi.setTrackStatus(JSON.stringify(str))
        }
      },
    },
  ])

  return ThinkingAnalyticsAPI
})()

/**
 * TDAnalytics, ThinkingData Analytics SDK for Mini Game & App.
 * @example
 * //引入SDK
 * var TDAnalytics = require('./thinkingdata.wx.min.js');
 * //初始化SDK
 * var config = {
 *   appId: 'your-app-id', // 项目的 App ID
 *   serverUrl: 'https://your.serverurl.com' // 数据上报地址
 * };
 * TDAnalytics.init(config);
 * //用户登录
 * TDAnalytics.login('thinker');
 * //设置事件公共属性
 * var superProperties = {
 *     channel : 'td', //字符串
 *     age : 1,//数字
 *     isSuccess : true,//布尔
 *     birthday :  new Date(),//日期
 *     array : [ 'value' ],//数组
 *     row : { key : 'value' },//对象
 *     array_rows : [ { key : 'value' } ]//对象组
 * };
 * TDAnalytics.setSuperProperties(superProperties);
 * //上报事件
 * var eventProperties = {
 *     product_name: '钻石'
 * };
 * TDAnalytics.track({
 *     eventName: 'product_buy', // 事件名称
 *     properties: eventProperties //事件属性
 * });
 * //上报用户属性
 * var userProperties = {
 *     username: 'tiki'
 * };
 * TDAnalytics.userSet({
 *     properties: userProperties
 * });
 */

var TDAnalytics = /*#__PURE__*/ (function () {
  function TDAnalytics() {
    _classCallCheck(this, TDAnalytics)
  }

  _createClass(TDAnalytics, null, [
    {
      key: '_shareInstance',
      // constructor(){
      // }
      value: function _shareInstance(appId) {
        if (this._instanceMaps[appId] !== undefined) {
          return this._instanceMaps[appId]
        } else if (this._defaultInstance !== undefined) {
          return this._defaultInstance
        } else {
          return undefined
        }
      }, //初始化

      /**
       * Create a new instance.
       * All properties that can be set are independent of the main instance.
       *
       * @param {Object} config
       * @param {String} config.appId Project App ID
       * @param {String} config.serverUrl Project Server Url
       * @param {Object} config.autoTrack Auto-tracking Events
       * @param {Boolean} config.autoTrack.appShow Auto Track App Show Events
       * @param {Boolean} config.autoTrack.appHide Auto Track App Hide Events
       * @param {Boolean} config.enableLog Enable Log Printing
       */
    },
    {
      key: 'init',
      value: function init(config) {
        var td = new ThinkingAnalyticsAPI(config)
        td.init()

        if (td !== undefined) {
          if (this._defaultInstance === undefined) {
            this._defaultInstance = td
            this._instanceMaps = {}
          }

          this._instanceMaps[config.appId] = td
        }
      }, //轻实例

      /**
       * Get sub-instance with name
       *
       * @param {String} appId Project App ID
       * @returns {String} Sub-instance token
       */
    },
    {
      key: 'lightInstance',
      value: function lightInstance(appId) {
        return this._shareInstance(appId).lightInstance()
      }, //事件
      //普通事件

      /**
       * Track a narmal Event.
       * @param {Object} options
       * @param {String} options.eventName Event name, required
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'track',
      value: function track() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).track(options.eventName, options.properties, options.time, options.onComplete)
      },
      /**
       * Track a first Event
       * @param {Object} options event infomations
       *
       * @param {String} options.eventName Event name, required
       * @param {String} options.firstCheckId Event ID, to mark the event, default is #device_id, required
       * @param {Date} options.time Event time, optional
       * @param {Object} options.properties Event properties, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID, optional
       */
    },
    {
      key: 'trackFirst',
      value: function trackFirst() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).trackFirstEvent(options)
      },
      /**
       * Track a updatable Event
       * @param {Object} options event infomations
       *
       * @param {String} options.eventName Event name, required
       * @param {String} options.eventId Event ID, to mark the event, required
       * @param {Date} options.time Event time, optional
       * @param {Object} options.properties Event properties, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID, optional
       */
    },
    {
      key: 'trackUpdate',
      value: function trackUpdate() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).trackUpdate(options)
      },
      /**
       * Track a overwritable Event
       * @param {Object} options event infomations
       *
       * @param {String} options.eventName Event name, required
       * @param {String} options.eventId Event ID, to mark the event, required
       * @param {Date} options.time Event time, optional
       * @param {Object} options.properties Event properties, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID, optional
       */
    },
    {
      key: 'trackOverwrite',
      value: function trackOverwrite() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).trackOverwrite(options)
      },
      /**
       * Record the event duration, call this method to start the timing, stop the timing when the target event is uploaded, and add the attribute #duration to the event properties, in seconds.
       * @param {Object} options
       * @param {String} options.eventName Event name
       * @param {Date} options.time Event time
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'timeEvent',
      value: function timeEvent() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).timeEvent(options.eventName, options.time)
      },
      /**
       *
       */
      // static enableAutoTrack(eventType, properties, appid) {
      // }

      /**
       * Sets the user property, replacing the original value with the new value if the property already exists.
       * @param {Object} options
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userSet',
      value: function userSet() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userSet(options.properties, options.time, options.onComplete)
      },
      /**
       * Sets a single user attribute, ignoring the new attribute value if the attribute already exists.
       * @param {Object} options
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userSetOnce',
      value: function userSetOnce() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userSetOnce(options.properties, options.time, options.onComplete)
      },
      /**
       * Reset user properties.
       * @param {Object} options
       * @param {String} options.property Event property, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userUnset',
      value: function userUnset() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userUnset(options.property, options.time, options.onComplete)
      },
      /**
       * Adds the numeric type user attributes.
       * @param {Object} options
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userAdd',
      value: function userAdd() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userAdd(options.properties, options.time, options.onComplete)
      },
      /**
       * Append a user attribute of the List type.
       * @param {Object} options
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userAppend',
      value: function userAppend() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userAppend(options.properties, options.time, options.onComplete)
      },
      /**
       * The element appended to the library needs to be done to remove the processing,and then import.
       * @param {Object} options
       * @param {Object} options.properties Event properties, optional
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userUniqAppend',
      value: function userUniqAppend() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userUniqAppend(options.properties, options.time, options.onComplete)
      },
      /**
       * Delete the user attributes,This operation is not reversible and should be performed with caution.
       * @param {Object} options
       * @param {Date} options.time Event time, optional
       * @param {Function} options.onComplete Callback, optional
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'userDelete',
      value: function userDelete() {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {}
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).userDel(options.time, options.onComplete)
      },
      /**
       * Set the public event attribute, which will be included in every event uploaded after that. The public event properties are saved without setting them each time.
       * @param {Object} properties Public event attribute
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'setSuperProperties',
      value: function setSuperProperties(properties) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).setSuperProperties(properties)
      },
      /**
       * Clears a public event attribute.
       * @param {String} property Public event attribute key to clear
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'unsetSuperProperty',
      value: function unsetSuperProperty(property) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).unsetSuperProperty(property)
      },
      /**
       * Clear all public event attributes.
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'clearSuperProperties',
      value: function clearSuperProperties(appId) {
        this._shareInstance(appId).clearSuperProperties()
      },
      /**
       * Gets the public event properties that have been set.
       * @param {String} appId Project App ID
       * @returns {Object} Public event properties that have been set
       */
    },
    {
      key: 'getSuperProperties',
      value: function getSuperProperties(appId) {
        return this._shareInstance(appId).getSuperProperties()
      },
      /**
       * Set dynamic public properties. Each event uploaded after that will contain a public event attribute.
       * @param {Function} dynamicProperties Dynamic public attribute interface
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'setDynamicSuperProperties',
      value: function setDynamicSuperProperties(dynamicProperties) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).setDynamicSuperProperties(dynamicProperties)
      },
      /**
       * Gets prefabricated properties for all events.
       * @param {String} appId Project App ID
       * @returns {Object} preset properties
       */
    },
    {
      key: 'getPresetProperties',
      value: function getPresetProperties(appId) {
        return this._shareInstance(appId).getPresetProperties()
      },
      /**
       * Set the account ID. Each setting overrides the previous value. Login events will not be uploaded.
       * @param {String} accountId Login user account ID
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'login',
      value: function login(accountId) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).login(accountId)
      },
      /**
       * Clearing the account ID will not upload user logout events.
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'logout',
      value: function logout(appId) {
        this._shareInstance(appId).logout()
      },
      /**
       * Set the distinct ID to replace the default UUID distinct ID.
       * @param {String} distinctId Distinct ID
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'setDistinctId',
      value: function setDistinctId(distinctId) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).identify(distinctId)
      },
      /**
       * Get a visitor ID The #distinct_id value in the reported data.
       * @param {String} appId Project App ID
       * @returns {String} distinct ID
       */
    },
    {
      key: 'getDistinctId',
      value: function getDistinctId(appId) {
        return this._shareInstance(appId).getDistinctId()
      },
      /**
       * Get sdk version
       * @returns {String} sdk version
       */
    },
    {
      key: 'getSDKVersion',
      value: function getSDKVersion() {
        return Config.LIB_VERSION
      },
      /**
       * Get device ID
       * @param {String} appId Project App ID
       * @returns {String} Current Device ID
       */
    },
    {
      key: 'getDeviceId',
      value: function getDeviceId(appId) {
        return this._shareInstance(appId).getDeviceId()
      },
      /**
       * Empty the cache queue. When this api is called, the data in the current cache queue will attempt to be reported.
       * If the report succeeds, local cache data will be deleted.
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'flush',
      value: function flush(appId) {
        this._shareInstance(appId).flush()
      },
      /**
       * Set status for events reporting
       * PAUSE, pause events reporting
       * STOP, stop events reporting, and cache data will be cleared
       * SAVE_ONLY, event data stores in the cache, but not be reported (native support, js equal to NORMAL)
       * NORMAL, resume event reporting
       * @param {String} status Events reporting status
       * @param {String} appId Project App ID
       */
    },
    {
      key: 'setTrackStatus',
      value: function setTrackStatus(status) {
        var appId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ''

        this._shareInstance(appId).setTrackStatus(status)
      },
      /**
       * Get old api ThinkingDataAPI
       * @returns {Function} ThinkingDataAPI, old api
       */
    },
    {
      key: 'ThinkingDataAPI',
      value: function ThinkingDataAPI() {
        return ThinkingAnalyticsAPI
      },
    },
  ])

  return TDAnalytics
})()

export default TDAnalytics
